unit Unit1;

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QExtCtrls, QGrids, kdclxcore, kdinterrupt;

type
  TForm1 = class(TForm)
    GIRQ: TStringGrid;
    IRQ1CheckBox: TCheckBox;
    IRQ2CheckBox: TCheckBox;
    IRQ3CheckBox: TCheckBox;
    IRQ4CheckBox: TCheckBox;
    IRQ5CheckBox: TCheckBox;
    IRQ6CheckBox: TCheckBox;
    IRQ7CheckBox: TCheckBox;
    IRQ8CheckBox: TCheckBox;
    IRQ9CheckBox: TCheckBox;
    IRQ10CheckBox: TCheckBox;
    IRQ11CheckBox: TCheckBox;
    IRQ12CheckBox: TCheckBox;
    IRQ13CheckBox: TCheckBox;
    IRQ14CheckBox: TCheckBox;
    IRQ15CheckBox: TCheckBox;
    CommonIRQsBevel: TBevel;
    OpenButton: TButton;
    CloseButton: TButton;
    RegisterCardButton: TButton;
    UnregisterCardButton: TButton;
    CommentLabel1: TLabel;
    CommentLabel2: TLabel;
    CommentLabel3: TLabel;
    CommentLabel4: TLabel;
    ReadmeLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure IRQCheckBoxClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure RegisterCardButtonClick(Sender: TObject);
    procedure UnregisterCardButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  protected
    procedure ShowButtons;
  public
    { Public declarations }
  end;

type
  // A scheduling thread that monitors interrupt statistics
  TIntMonitoringThread = class (TThread)
  protected
    FGrid: TStringGrid;
    procedure Execute; override;
    procedure ShowInterrupts;
  public
    constructor Create(AGrid: TStringGrid);
    destructor Destroy; override;
  end;

const
  IntNum = 15;
    
var
  Form1: TForm1;
  InterruptsGenerated, InterruptsHandled: array [1..IntNum] of Integer;
  InterruptsPreviousTimes, InterruptsLatestTimes: array [1..IntNum] of Extended;


implementation
uses
  DataModuleUnit;

var
  IntMonitoringThread: TIntMonitoringThread;


{$R *.xfm}

procedure TIntMonitoringThread.ShowInterrupts;
var
  i: Integer;
begin
  for i:= 1 to IntNum do
  begin
    if Assigned(FGrid) then
      if (not (csDestroying in FGrid.ComponentState) and
        not (csDestroying in FGrid.Owner.ComponentState) ) then
    begin
      FGRid.Cells[1, i]:= IntToStr(InterruptsGenerated[i]);
      FGRid.Cells[2, i]:= IntToStr(InterruptsHandled[i]);
      FGrid.Cells[3, i]:= FloatToStrF(InterruptsLatestTimes[i] - InterruptsPreviousTimes[i], ffFixed, 6, 3);
    end;
  end;
end;

procedure TIntMonitoringThread.Execute;
begin
  repeat
    Synchronize (ShowInterrupts)
  until Terminated;
end;

constructor TIntMonitoringThread.Create;
begin
  inherited Create (False);
  FGrid:= AGrid;
  FreeOnTerminate:= True;
end;

destructor TIntMonitoringThread.Destroy;
begin
  FGrid:= nil;
  inherited;
end;

procedure TForm1.ShowButtons;
begin
  with CustomCardDataModule do
  begin
    OpenButton.Enabled:= not kdCustomCard1.ModuleLoaded;
    RegisterCardButton.Enabled:= (kdCustomCard1.ModuleLoaded) and (not kdCustomCard1.CardRegistered);
    UnregisterCardButton.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    CloseButton.Enabled:= (kdCustomCard1.ModuleLoaded) and (not kdCustomCard1.CardRegistered);
    IRQ1CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ2CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ3CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ4CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ5CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ6CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ7CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ8CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ9CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ10CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ11CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ12CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ13CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ14CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
    IRQ15CheckBox.Enabled:= (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  with CustomCardDataModule do
    kdCustomCard1.ModuleLoaded:= False;
  GIRQ.Cells[0, 0]:= 'Int #';
  GIRQ.Cells[1, 0]:= 'Generated';
  GIRQ.Cells[2, 0]:= 'Handled';
  GIRQ.Cells[3, 0]:= 'I-val(ms)';
  IntMonitoringThread:= TIntMonitoringThread.Create(GIRQ);

end;

procedure TForm1.OpenButtonClick(Sender: TObject);
begin
  with CustomCardDataModule.kdCustomCard1 do
  begin
    ModuleLoaded:= True;
    if ModuleLoaded = True then
      MessageDlg('The GDD is successfully loaded !', mtInformation, [mbOK], 0, mbOK)
    else
      MessageDlg('The GDD is not loaded. If you are a non-root, the GDD must be loaded while entering the system.',
        mtInformation, [mbOK], 0, mbOK);
    ShowButtons;
  end;
end;

procedure TForm1.IRQCheckBoxClick(Sender: TObject);
var
  i: Integer;
begin
  // a cleverer code would be welcome
  with CustomCardDataModule do
  begin
    if (kdCustomCard1.ModuleLoaded) and (kdCustomCard1.CardRegistered) then
    begin
      with kdCustomCard1, Sender do
        if TCheckBox(Sender).Checked then
        begin
          for i:= 0 to CardItems.Count - 1 do
            if TkdResource(CardItems[i]) is TkdInterrupt then
              if TkdInterrupt(CardItems[i]).InterruptNumber = TComponent(Sender).Tag then
                if not TkdInterrupt(CardItems[i]).EnableInterrupt then TCheckBox(Sender).Checked:= False
                  else begin Exit end;
          TCheckBox(Sender).Checked:= False;  // TkdInterrupt resource has not been found
        end else
        begin
          for i:= 0 to CardItems.Count - 1 do
            if TkdResource(CardItems[i]) is TkdInterrupt then
              if TkdInterrupt(CardItems[i]).InterruptNumber = TComponent(Sender).Tag then
                TkdInterrupt(CardItems[i]).DisableInterrupt;
        end;
    end;
  end;
end;

procedure TForm1.CloseButtonClick(Sender: TObject);
begin
  with CustomCardDataModule.kdCustomCard1 do
  begin
   ModuleLoaded:= False;
   if ModuleLoaded = False then
   MessageDlg('The GDD has been successfully unloaded !', mtInformation, [mbOK], 0, mbOK);
   ShowButtons;
  end;
end;

procedure TForm1.RegisterCardButtonClick(Sender: TObject);
begin
  with CustomCardDataModule.kdCustomCard1 do
  begin
    RegisterCard;
    if CardRegistered then
      MessageDlg('Your card is successfully registered !', mtInformation, [mbOK], 0, mbOK)
    else MessageDlg('Problem registering your card, maybe too many interrupt handling components! Try to leave only necessary ones.', mtInformation, [mbOK], 0, mbOK);
    ShowButtons;
  end;
end;

procedure TForm1.UnregisterCardButtonClick(Sender: TObject);
begin
  with CustomCardDataModule.kdCustomCard1 do
  begin
    UnregisterCard;
    if not CardRegistered then
    MessageDlg('Your card has been successfully unregistered !', mtInformation, [mbOK], 0, mbOK);
    ShowButtons;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  IntMonitoringThread.Terminate;
  //  error -> IntMonitoringThread.WaitFor;
end;

end.
