unit Altera_Lib;

interface
uses
  kdclxcore,
  kdinterrupt;

const
  ALTERA_MODE_BYTE = 0;
  ALTERA_MODE_WORD = 1;
  ALTERA_MODE_DWORD = 2;

var
  ALTERA_DEFAULT_VENDOR_ID: LongWord = $1172; // ALTERA
  ALTERA_DEFAULT_DEVICE_ID: LongWord = $0004;

  ALTERA_REG_DMACSR: Byte = $0;
  ALTERA_REG_DMAACR: Byte = $4;
  ALTERA_REG_DMABCR: Byte = $8;
  ALTERA_REG_DMAISR: Byte = $c;
  ALTERA_REG_DMALAR: Byte = $10;
  ALTERA_REG_SDRAM_BAR: Byte = $18;
  ALTERA_REG_SDRAM_CFG0: Byte = $1c;
  ALTERA_REG_SDRAM_CFG1: Byte = $20;
  ALTERA_REG_SDRAM_CFG2: Byte = $24;
  ALTERA_REG_TARG_PERF: Byte = $80;
  ALTERA_REG_MSTR_PERF: Byte = $84;


  INTERRUPT_ENABLE: Word = $0001;
  DMA_ENABLE: Word =  $0010;
  TRXCOMPINTDIS: Word = $0020;
  CHAINEN: Word = $0100;
  DIRECTION: Word = $0008;


  FIFO_BASE_OFFSET: LongWord = 512 * 1024;
  SDRAM_BASE_OFFSET: Byte = 0;
  ALTERA_DMA_FIFO_REGS: Byte = 128;

  INTERUPT_PENDING: Byte = $01;
  ERROR_PENDING: Byte = $02;
  LOCAL_REQ: Byte = $04;
  TX_COMP: LongWord = $08;
  ACR_LOADED: Byte = $10;
  CHAIN_STARTED: Byte = $20;

  ALTERA_ErrorString: string;

  // main
  function ALTERA_CountCards(dwVendorId, dwDeviceId: LongWord; AuxDevicePath: String): LongWord;
  function ALTERA_Open(ALTERACard: TkdCustomCard; dwVendorId, dwDeviceId: LongWord; nCardNum: Byte): Boolean;
  procedure ALTERA_Close(ALTERACard: TkdCustomCard);

  // general rear/write function
  procedure ALTERA_ReadWriteBlock(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord;
    fRead: Boolean; buf: Pointer; dwBytes: LongWord; mode: Byte);
  function ALTERA_ReadByte(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord): Byte;
  function ALTERA_ReadWord(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord): Word;
  function ALTERA_ReadDword(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord): LongWord;
  procedure ALTERA_WriteByte(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord; data: Byte);
  procedure ALTERA_WriteWord(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord; data: Word);
  procedure ALTERA_WriteDword(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord; data: LongWord);

  // handle interrupts
  function ALTERA_GetIntPrompt(ALTERACard: TkdCustomCard): String;
  function ALTERA_IntIsEnabled(ALTERACard: TkdCustomCard): Boolean;
  function ALTERA_IntEnable(ALTERACard: TkdCustomCard; Handler: TkdInterruptHandler): Boolean;
  procedure ALTERA_IntDisable(ALTERACard: TkdCustomCard);

  // dma operations introduced but not used in the diagnostic program
  function ALTERA_DMAReadWriteBlock (ALTERACard: TkdCustomCard; dwLocalAddr: LongWord;
    pBuffer: Pointer; fRead: Boolean;  dwBytes: LongWord; fChained: Boolean): Boolean;

implementation
uses
  SysUtils,
  Classes,
  kdframeworkassistant,
  kdranges,
  kdregister,
  kddmabuffer,
  kdpcihdr;
  
  function ALTERA_DetectCardElements(ALTERACard: TkdCustomCard): Boolean; forward;
  
  function ALTERA_CountCards(dwVendorId, dwDeviceId: LongWord; AuxDevicePath: String): LongWord;
  var
    FA: TkdFrameworkAssistant;
  begin    
    Result:= 0;
    SetLength(ALTERA_ErrorString, 0);
    FA:= TkdFrameworkAssistant.Create(AuxDevicePath);
    try
      with FA do
      begin
        if not EnumeratePciCards then
	begin
	  ALTERA_ErrorString:= 'Failed enumerating PCI cards';
	  Exit;
	end;
        PciVid:= dwVendorId;
        PciDid:= dwdeviceId;
        Result:= PciVidDidCount;  { get number of cards by PciVid:PciDid specified above }
      end;
    finally
      FA.Free;
    end;
    if Result = 0 then
      ALTERA_ErrorString:= 'no cards found';
  end;

  procedure ALTERA_SpecifyInterruptHandling (ALTERACard: TkdCustomCard);
  var
    i: Integer;
  begin
  	{ -------------------------------------------------------------------- }
	{ ATTENTION! }
        { This is a sample of handling interrupts: }
        { Our InterruptHandler property's body below is to cancel the source of interrupt. }
        { Otherwise, the PC will hang when an interrupt occurs }
        { You will need to modify this code to fit your specific ALTERA card. }
        { You will be hinted with *TODO* statements through out the code underneath }

        { 1st of all, add yet 1 additional resource - register to acknowledge card's interrupt }
        ALTERACard.AddResource(TkdHWRangeRegister.Create(ALTERACard.Owner));
        { give the register some name for reference }
        TkdHWRangeRegister(ALTERACard.CardItems[ALTERACard.Count - 1]).Name:= 'AckRegister';

        { now we must bind this register to appropriate address range
        { and specify the offset of register's location within the range : }
        { e.g. 1: TkdHWRangeRegister(ALTERACard.CardItems[ALTERACard.Count - 1]).HWRange:= ... }
        { e.g. 2: TkdHWRangeRegister(ALTERACard.CardItems[ALTERACard.Count - 1]).Offset:= ... }

        { 1. we bind first-found range (*TODO*: MODIFY this code for YOUR particular card) }
        for i:= 0 to ALTERACard.Count - 2 do
          if TComponent(ALTERACard.CardItems[i]) is TkdHWRange then
          begin
            TkdHWRangeRegister(ALTERACard.CardItems[ALTERACard.Count - 1]).HWRange:=
              TkdHWRange(ALTERACard.CardItems[i]);
            Break; // stop enumeration
          end;

        { 2. we specify offset 0 within the range (*TODO*: MODIFY this code for YOUR particular card) }
        TkdHWRangeRegister(ALTERACard.CardItems[ALTERACard.Count - 1]).Offset:= 0;
        { by the way, specify size of data to acknowledge = (DWORD)};
        TkdHWRangeRegister(ALTERACard.CardItems[ALTERACard.Count - 1]).Size:= rsDword;


        { 3. now prepare our interrupt component to acknowledge hardware interrupts
             with the help of our register }
	for i:= 0 to ALTERACard.Count - 1 do
	  if TComponent(ALTERACard.CardItems[i]) is TkdInterrupt then
	  begin
            // kernel-mode int.handler acknowledgement turned on
            TkdInterrupt (ALTERACard.CardItems[i]).UseInterruptHandler:= True;
            // this is just for console output safety
            TkdInterrupt (ALTERACard.CardItems[i]).InterruptRTSynchronized:= False;

            { store 0 to virtual accumulator: 'and 0' }
	    TkdInterrupt (ALTERACard.CardItems[i]).InterruptHandler.Add('and 0');
            { write this 0 to register by name, so cancel the source of interrupt }
            TkdInterrupt (ALTERACard.CardItems[i]).InterruptHandler.Add('write AckRegister');
            { SOI action delivers interrupt towards a user-mode interrupt handler }
	    TkdInterrupt (ALTERACard.CardItems[i]).InterruptHandler.Add('SOI');
	    Break;
	  end;
	{ -------------------------------------------------------------------- }
  end;

  procedure ALTERA_CreateAndAddDMABufferResource(ALTERACard: TkdCustomCard);
  var
    i: Integer;
    DMABuffer: TkdDMABuffer;
  begin
    DMABuffer:= TkdDMABuffer.Create(ALTERACard.Owner);
    ALTERACard.AddResource(DMABuffer);
    TkdDMABuffer(ALTERACard.CardItems[ALTERACard.Count -1]).Name:= 'DMABuffer';
    { Now, because we will use a user-allocated buffer DMA for a concrete PCI ALTERA }
    { DMABuffer component bust me obligately provided with the PCI info. }
    { Consequently, PCI configuration must be supplied to PCIConfig property }

    { locate PCIConfig, and assign it to DMA}
    for i:= 0 to ALTERACard.Count - 1 do
      if TComponent(ALTERACard.CardItems[i]) is TkdPCIConfig then
      begin
        DMABuffer.PCIConfig:= TkdPCINonBridge(ALTERACard.CardItems[i]);
        Break;
      end;
  end;

  function ALTERA_Open(ALTERACard: TkdCustomCard; dwVendorId, dwDeviceId: LongWord; nCardNum: Byte): Boolean;
  var
    FA: TkdFrameworkAssistant;
    label _exit_;
  begin
    Result:= False;
    SetLength(ALTERA_ErrorString, 0);
    FA:= TkdFrameworkAssistant.Create(ALTERACard.DevicePath+'/'+aux_device_file_name);
    try
      with FA do
      begin
        if not EnumeratePciCards then
	begin
	  ALTERA_ErrorString:= 'Failed enumerating PCI cards';
	  Exit;
	end;
	
        PciVid:= dwVendorId;
        PciDid:= dwdeviceId;
	{ search number of cards by PciVid, PciDid values specified }
        { ensure that at least one card found }
        if PciVidDidCount = 0 then
	begin
	  ALTERA_ErrorString:= 'Could not find PCI card';
	  Goto _exit_;
	end;

	if PciVidDidCount < nCardNum then
	begin
	  ALTERA_ErrorString:= 'Card out of range of available cards';
	  Goto _exit_;
	end;
	
	if PciVidDidConfig [nCardNum] = nil then
	begin
	  ALTERA_ErrorString:= 'Could not get actual configuration for card';
	  Goto _exit_;
	end;
	
	{ Print choosen card info first: }
	Writeln (Format('%s', [PrintPciCardInfo(PciVidDidConfig [nCardNum])]));
	{ Make up this card }
	ConstructPciCardResources(PciVidDidConfig [nCardNum], ALTERACard);

        {Specify code that will handle your ALTERA card's interrupts }
        ALTERA_SpecifyInterruptHandling (ALTERACard);

        // we could test DMA operation as well. Diagnostic program does not use it.
        ALTERA_CreateAndAddDMABufferResource (ALTERACard);

	{ Finally, register the card itself - all things will be taken in action }
	ALTERACard.RegisterCard;
	if not ALTERACard.CardRegistered then
	begin
	  ALTERA_ErrorString:= 'Failed registering card. Resource failed: '+
	    ALTERACard.FailedResource.Name;
	  Goto _exit_;
	end;

	if not ALTERA_DetectCardElements(ALTERACard) then
	begin
	  ALTERA_ErrorString:= 'Card does not have all items expected for ALTERA';
	  Goto _exit_;
	end;

	Result:= True;
	Exit;

_exit_:
        { optionally unregister for new test }
        if ALTERACard.CardRegistered then ALTERACard.UnregisterCard;
	{ optionally clear all resources to use the card object for new test }
	ALTERACard.Clear;
      end;
    finally
      FA.Free;
    end;
  end;

  procedure ALTERA_Close(ALTERACard: TkdCustomCard);
  begin
    ALTERACard.UnregisterCard;
  end;


  function ALTERA_DetectCardElements(ALTERACard: TkdCustomCard): Boolean;
  var
    i: Integer;
  begin
    Result:= False;
    { check that at least one memory or port space is located on the card }
    for i:= 0 to ALTERACard.Count - 1 do
      if TComponent(ALTERACard.CardItems[i]) is TkdHWRange then
      begin
        Result:= True;
	Break;
      end;
  end;


  function GetHWRangeByIndex(ALTERACard: TkdCustomCard; Index: Byte): TkdHWRange;
  var
    i: Integer;
    SearchIndex: Byte;
  begin
    Result:= nil;
    SearchIndex:= 1;
    for i:= 0 to ALTERACard.Count do
      if TComponent(ALTERACard.CardItems[i]) is TkdHWRange then
        if SearchIndex = Index then
        begin
          Result:= TkdHWRange(ALTERACard.CardItems[i]);
          Break
        end else Inc(SearchIndex);
    if Result = nil then
      Writeln (Format('Address space BAR%d not found on board',[Index-1]));
  end;

  procedure ALTERA_ReadWriteBlock(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord;
    fRead: Boolean; buf: Pointer; dwBytes: LongWord; mode: Byte);

  function Mode2Bytes: Byte;
  begin
    if mode = ALTERA_MODE_BYTE then
      Result:= 1 else
        if mode = ALTERA_MODE_WORD then
          Result:= 2 else
            Result:= 4;
  end;
  
  var
    AHWRange: TkdHWRange;
  begin
    AHWRange:= GetHWRangeByIndex(ALTERACard, address_space);
    if not Assigned (AHWRange) then Exit;
    
    if AHWRange is TkdPortRange then
    begin
       TkdPortRange(AHWRange).TransferOffset:= dwOffset;
       TkdPortRange(AHWRange).TransferAutoInc:= True;
       TkdPortRange(AHWRange).TransferCount:= dwBytes div Mode2Bytes;
       if mode = ALTERA_MODE_BYTE then
         TkdPortRange(AHWRange).TransferSize:= tsByte else
           if mode = ALTERA_MODE_WORD then
             TkdPortRange(AHWRange).TransferSize:= tsWord else
               TkdPortRange(AHWRange).TransferSize:= tsDWord;
       if fRead then
         TkdPortRange(AHWRange).TransferDir:= tdRead else
           TkdPortRange(AHWRange).TransferDir:= tdWrite;

       TkdPortRange(AHWRange).Transfer(buf, Mode2Bytes*TkdPortRange(AHWRange).TransferCount);
    end else
    begin
       TkdMemoryRange(AHWRange).TransferOffset:= dwOffset;
       TkdMemoryRange(AHWRange).TransferAutoInc:= True;
       TkdMemoryRange(AHWRange).TransferCount:= dwBytes div Mode2Bytes;
       if mode = ALTERA_MODE_BYTE then
         TkdMemoryRange(AHWRange).TransferSize:= tsByte else
           if mode = ALTERA_MODE_WORD then
             TkdMemoryRange(AHWRange).TransferSize:= tsWord else
               TkdMemoryRange(AHWRange).TransferSize:= tsDWord;
       if fRead then
         TkdMemoryRange(AHWRange).TransferDir:= tdRead else
           TkdMemoryRange(AHWRange).TransferDir:= tdWrite;

       TkdMemoryRange(AHWRange).Transfer(buf, Mode2Bytes*TkdMemoryRange(AHWRange).TransferCount);
    end;
  end;

  function ALTERA_ReadByte(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord): Byte;
  var
    AHWRange: TkdHWRange;
    data: Byte;
    LinearAddr: ^Byte;
  begin
    Result:= 0;
    AHWRange:= GetHWRangeByIndex(ALTERACard, address_space);
    if not Assigned (AHWRange) then Exit;

    if AHWRange is TkdPortRange then
      ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, True, @data, SizeOf(Byte), ALTERA_MODE_BYTE) else
        if not Assigned (TkdMemoryRange(AHWRange).DirectRange) then
          ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, True, @data, SizeOf(Byte), ALTERA_MODE_BYTE) else
          begin
            // apply ability to direct access memory range
            LinearAddr:= Pointer(LongWord(TkdDirectRPIOMRange(TkdMemoryRange(AHWRange).DirectRange).UserDirectAddr)+dwOffset);
            data:= LinearAddr^;
          end;

    Result:= data;
  end;

  function ALTERA_ReadWord(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord): Word;
  var
    AHWRange: TkdHWRange;
    data: Word;
    LinearAddr: ^Word;
  begin
    Result:= 0;
    AHWRange:= GetHWRangeByIndex(ALTERACard, address_space);
    if not Assigned (AHWRange) then Exit;
    
    if AHWRange is TkdPortRange then
      ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, True, @data, SizeOf(Word), ALTERA_MODE_WORD) else
        if not Assigned (TkdMemoryRange(AHWRange).DirectRange) then
          ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, True, @data, SizeOf(Word), ALTERA_MODE_WORD) else
          begin
            // apply ability to direct access memory range
            LinearAddr:= Pointer(LongWord(TkdDirectRPIOMRange(TkdMemoryRange(AHWRange).DirectRange).UserDirectAddr)+dwOffset);
            data:= LinearAddr^;
          end;

    Result:= data;
  end;

  function ALTERA_ReadDword(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord): LongWord;
  var
    AHWRange: TkdHWRange;
    data: LongWord;
    LinearAddr: ^LongWord;
  begin
    Result:= 0;
    AHWRange:= GetHWRangeByIndex(ALTERACard, address_space);
    if not Assigned (AHWRange) then Exit;

    if AHWRange is TkdPortRange then
      ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, True, @data, SizeOf(LongWord), ALTERA_MODE_DWORD) else
        if not Assigned (TkdMemoryRange(AHWRange).DirectRange) then
          ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, True, @data, SizeOf(LongWord), ALTERA_MODE_DWORD) else
          begin
            // apply ability to direct access memory range
            LinearAddr:= Pointer(LongWord(TkdDirectRPIOMRange(TkdMemoryRange(AHWRange).DirectRange).UserDirectAddr)+dwOffset);
            data:= LinearAddr^;
          end;
    Result:= data;
  end;

  procedure ALTERA_WriteByte(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord; data: Byte);
  var
    AHWRange: TkdHWRange;
    LinearAddr: ^Byte;
  begin
    AHWRange:= GetHWRangeByIndex(ALTERACard, address_space);
    if not Assigned (AHWRange) then Exit;

    if AHWRange is TkdPortRange then
      ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, False, @data, SizeOf(Byte), ALTERA_MODE_BYTE) else
        if not Assigned (TkdMemoryRange(AHWRange).DirectRange) then
          ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, False, @data, SizeOf(Byte), ALTERA_MODE_BYTE) else
          begin
            // apply ability to direct access memory range
            LinearAddr:= Pointer(LongWord(TkdDirectRPIOMRange(TkdMemoryRange(AHWRange).DirectRange).UserDirectAddr)+dwOffset);
            LinearAddr^:= data;
          end;
  end;

  procedure ALTERA_WriteWord(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord; data: Word);
  var
    AHWRange: TkdHWRange;
    LinearAddr: ^Word;
  begin
    AHWRange:= GetHWRangeByIndex(ALTERACard, address_space);
    if not Assigned (AHWRange) then Exit;
    
    if AHWRange is TkdPortRange then
      ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, False, @data, SizeOf(Word), ALTERA_MODE_WORD) else
        if not Assigned (TkdMemoryRange(AHWRange).DirectRange) then
          ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, False, @data, SizeOf(Word), ALTERA_MODE_WORD) else
          begin
            // apply ability to direct access memory range
            LinearAddr:= Pointer(LongWord(TkdDirectRPIOMRange(TkdMemoryRange(AHWRange).DirectRange).UserDirectAddr)+dwOffset);
            LinearAddr^:= data;
          end;
  end;

  procedure ALTERA_WriteDword(ALTERACard: TkdCustomCard; address_space: Byte; dwOffset: LongWord; data: LongWord);
  var
    AHWRange: TkdHWRange;
    LinearAddr: ^LongWord;
  begin
    AHWRange:= GetHWRangeByIndex(ALTERACard, address_space);
    if not Assigned (AHWRange) then Exit;
    
    if AHWRange is TkdPortRange then
      ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, False, @data, SizeOf(LongWord), ALTERA_MODE_DWORD) else
        if not Assigned (TkdMemoryRange(AHWRange).DirectRange) then
          ALTERA_ReadWriteBlock(ALTERACard, address_space, dwOffset, False, @data, SizeOf(LongWord), ALTERA_MODE_DWORD) else
          begin
            // apply ability to direct access memory range
            LinearAddr:= Pointer(LongWord(TkdDirectRPIOMRange(TkdMemoryRange(AHWRange).DirectRange).UserDirectAddr)+dwOffset);
            LinearAddr^:= data;
          end;
  end;

  function ALTERA_GetIntPrompt(ALTERACard: TkdCustomCard): String;
  var
    i: Integer;
  begin
    SetLength (Result, 0);
    { search for interrupt resource }
    for i:= 0 to ALTERACard.Count - 1 do
      if TComponent (ALTERACard.CardItems[i]) is TkdInterrupt then
      begin
        if TkdInterrupt (ALTERACard.CardItems[i]).Enabled then
          Result:= 'Disable' else
            Result:= 'Enable';
        Break;
      end;
    if Length (Result) = 0 then Result:= 'Interrupt resource not found on board';
  end;

  function ALTERA_IntIsEnabled(ALTERACard: TkdCustomCard): Boolean;
  begin
    if ALTERA_GetIntPrompt(ALTERACard) = 'Disable' then
      Result:= True else Result:= False;
  end;

  function ALTERA_IntEnable(ALTERACard: TkdCustomCard; Handler: TkdInterruptHandler): Boolean;
  var
    i: Integer;
  begin
    Result:= False;
    for i:= 0 to ALTERACard.Count - 1 do
      if TComponent (ALTERACard.CardItems[i]) is TkdInterrupt then
      begin
        with TkdInterrupt (ALTERACard.CardItems[i]) do
        begin
          OnHwInterrupt:= Handler;
          EnableInterrupt;
          if Enabled then Result:= True;
        end;
      end;
  end;

  procedure ALTERA_IntDisable(ALTERACard: TkdCustomCard);
  var
    i: Integer;
  begin
    for i:= 0 to ALTERACard.Count - 1 do
      if TComponent (ALTERACard.CardItems[i]) is TkdInterrupt then
      begin
        TkdInterrupt (ALTERACard.CardItems[i]).DisableInterrupt;
        Break;
      end
  end;

  function ALTERA_DMAWait(ALTERACard: TkdCustomCard): Boolean;
  var
    fOK: Boolean;
    i: LongWord;
    dwDMAISR: LongWord;
  begin
    fOk:= False;
    i:= 10 * 1000 * (1000 div 2);  // wait 10 seconds - each loop waits 2 microseconds
    repeat
      Dec(i);

      (*pass BAR0 - this is number 1 among all TkdHWRange resources*)
      dwDMAISR:= ALTERA_ReadDword(ALTERACard, 1, ALTERA_REG_DMAISR);
      if (dwDMAISR and TX_COMP) > 0 then
      begin
        fOk:= True;
        Break
      end;
      if (dwDMAISR and ERROR_PENDING) > 0 then
      begin
        ALTERA_ErrorString:= 'hardware dma failure';
        Break
      end;

      with ALTERACard do
      begin
        SleepOptions:= [];
        SleepMicrosecs:= 2;
        Sleep;
      end;
    until i > 0; // condition for timeout only
    if i = 0 then
      ALTERA_ErrorString:= 'dma transfer timeout';
    Result:= fOk;
  end;

  function ALTERA_DMAReadWriteBlock (ALTERACard: TkdCustomCard; dwLocalAddr: LongWord;
    pBuffer: Pointer; fRead: Boolean;  dwBytes: LongWord; fChained: Boolean): Boolean;
  var
    fOk: Boolean;
    DMACsr: LongWord;
    i: LongWord;
    DMABuffer: TkdDMABuffer;
    dwTotalBytes: LongWord;
  label
    _exit_;  
  begin
    fOk:= False;
    if dwBytes = 0 then
    begin
      Result:= fOk; // false
      Exit;
    end;

    DMABuffer:= nil;
    // find DMA buffer resource on card and prepare for lock
    for i:= 0 to ALTERACard.Count - 1 do
      if TComponent(ALTERACard.CardItems[i]) is TkdDMABuffer then
      begin
        DMABuffer:= TkdDMABuffer(ALTERACard.CardItems[i]);
        DMABuffer.Size:= dwBytes;
        if fRead then
          DMABuffer.Options:= DMABuffer.Options + [boREAD_FROM_DEVICE]
        else
          DMABuffer.Options:= DMABuffer.Options + [boWRITE_TO_DEVICE];
        Break;  // everything is ok - we go on
      end;

    if not Assigned (DMABuffer) then
    begin
      Writeln ('DMA buffer resource not defined on board !');
      Result:= fOk; // false
      Exit;
    end;
    
    if not DMABuffer.Lock(pBuffer) then
    begin
      ALTERA_ErrorString:= 'cannot lock down buffer - error.';
      Result:= fOk;  // false
      Exit;
    end;

    if fRead then DMACsr:= DIRECTION else DMACsr:= 0;

    DMACsr:= DMACsr or DMA_ENABLE or TRXCOMPINTDIS or INTERRUPT_ENABLE;

    if DMABuffer.Pages > ALTERA_DMA_FIFO_REGS then
    begin
      ALTERA_ErrorString:= 'too many pages for chain transfer';
      // no enough pages
      goto _exit_;
    end;

    if fChained then
    begin
      DMACsr:= DMACsr or CHAINEN;
      for i:= 0 to DMABuffer.Pages - 1 do
      begin
        ALTERA_WriteDword (ALTERACard, 1 (*Bar0*), FIFO_BASE_OFFSET, LongWord(DMABuffer.Page[i].PhysicalAddr));
        ALTERA_WriteDword (ALTERACard, 1 (*Bar0*), FIFO_BASE_OFFSET, LongWord(DMABuffer.Page[i].Size))
      end;
      ALTERA_WriteDword (ALTERACard, 1(*Bar0*), ALTERA_REG_DMALAR, dwLocalAddr);
      ALTERA_WriteDword (ALTERACard, 1(*Bar0*), ALTERA_REG_DMACSR, DMACsr);  // this starts the DMA
      fOk:= ALTERA_DMAWait(ALTERACard);
    end else
    begin
      dwTotalBytes:= 0;
      ALTERA_WriteDword (ALTERACard, 1(*Bar0*), ALTERA_REG_DMACSR, DMACsr);
      for i:= 0 to DMABuffer.Pages - 1 do
      begin
        ALTERA_WriteDword (ALTERACard, 1(*Bar0*), ALTERA_REG_DMALAR, dwLocalAddr+dwTotalBytes);
        ALTERA_WriteDword (ALTERACard, 1(*Bar0*), ALTERA_REG_DMABCR, LongWord(DMABuffer.Page[i].Size));
        ALTERA_WriteDword (ALTERACard, 1(*Bar0*), ALTERA_REG_DMAACR, DMABuffer.Page[i].PhysicalAddr);
        fOk:= ALTERA_DMAWait(ALTERACard);
        if not fOk then
          Break;
        dwTotalBytes:= dwTotalBytes + DMABuffer.Page[i].Size;
      end;
    end;

  _exit_:
    DMABuffer.Unlock;
    Result:= fOk;
  end;
end.
