program Altera_Project;

{$APPTYPE CONSOLE}

uses
  Classes,
  SysUtils,
  kdclxcore,
  kdranges,
  kdinterrupt,
  kdregister,
  utils in '../utils.pas',
  Altera_Lib in 'Altera_Lib.pas';

type
  TALTERAUserModeInterruptHandler = class
  public
    procedure ALTERA_IntHandlerRoutine (Sender: TkdInterrupt);
  end;

var
  ALTERADataModule: TDataModule;
  ALTERACard: TkdCustomCard;
  cmd: Integer;
  line: String[255];
  lobres: Boolean = False;
  ALTERAUserModeInterruptHandler: TALTERAUserModeInterruptHandler;


procedure TALTERAUserModeInterruptHandler.ALTERA_IntHandlerRoutine (Sender: TkdInterrupt);
begin
  Writeln ('Got Int number ', Sender.InterruptsDirected, ' (Press Enter to view menu)');
end;

function ALTERA_GetAddrSpacePromptByIndex(ALTERACard: TkdCustomCard; Index: Byte): PChar;
var
  i: integer;
  SearchIndex: Byte;
begin
  Result:=  '';
  SearchIndex:= 1;
  for i:= 0 to ALTERACard.Count do
    if TComponent(ALTERACard.CardItems[i]) is TkdHWRange then
      if SearchIndex = Index then
      begin
        if TComponent(ALTERACard.CardItems[i]) is TkdMemoryRange then
          Result:= PChar(Format('%s (%s-%s)',[TComponent(ALTERACard.CardItems[i]).Name,
            TkdMemoryRange(ALTERACard.CardItems[i]).RangeFrom, TkdMemoryRange(ALTERACard.CardItems[i]).RangeTo]))
        else
          Result:= PChar(Format('%s (%s-%s)',[TComponent(ALTERACard.CardItems[i]).Name,
            TkdPortRange(ALTERACard.CardItems[i]).RangeFrom, TkdPortRange(ALTERACard.CardItems[i]).RangeTo]));
        Break;
      end else Inc(SearchIndex);

  if Result = '' then
    Writeln (Format('Address space BAR%d not found on board',[Index-1]));
end;

function ALTERA_GetActiveMode(ad_mode: Byte): PChar;
begin
  if ad_mode = ALTERA_MODE_BYTE then Result:= 'BYTE (8 bit)' else
    if ad_mode = ALTERA_MODE_WORD then Result:= 'WORD (16 bit)' else
      Result:= 'DWORD (32 bit)'
end;

procedure ALTERA_AccessRanges (ALTERACard: TkdCustomCard);
var
  i: Integer;
  cmd, cmd2, cnt: Integer;
  ad_sp: Byte;
  ad_mode: Byte;
  addr, data: LongWord;
begin
  i:= 0;
  while i <= ALTERACard.Count - 1 do
  begin
    if TComponent(ALTERACard.CardItems[i]) is TkdHWRange then
      Break;
    Inc(i);
  end;
  if i = ALTERACard.Count then
  begin
    Writeln ('No active memory or port ranges on board');
    Exit;
  end;
  
  ad_sp:= 1;
  ad_mode:= ALTERA_MODE_DWORD;
  
  repeat
    Writeln;
    Writeln ('Access the board''s memory and port ranges');
    Writeln ('-----------------------------------------');
    Writeln (Format('1. Change active space: %s',[ALTERA_GetAddrSpacePromptByIndex(ALTERACard, ad_sp)]));
    Writeln (Format('2. Toggle active mode: %s',[ALTERA_GetActiveMode(ad_mode)]));
    Writeln ('3. Read from board');
    Writeln ('4. Write to board');
    Writeln ('99. Back to main menu');
    Writeln;
    Write ('Enter option: ');
    Readln(line);
    cmd:= StrToIntDef(line, 0);
    case cmd of
         1: begin
              Writeln;
              Writeln ('Choose memory or port space:');
              Writeln ('----------------------------');
              cnt:= 1;
              for i:= 0 to ALTERACard.Count - 1 do
                if TComponent(ALTERACard.CardItems[i]) is TkdHWRange then
                begin
                  Writeln (Format('%d. %s',[cnt, ALTERA_GetAddrSpacePromptByIndex(ALTERACard, cnt)]));
                  Inc (cnt);
                end;
              Writeln;
              Write ('Enter option: ');
              Readln(line);
              cmd2:= StrToIntDef(line, 0);
              if cmd2 in [1 .. cnt - 1] then ad_sp:= cmd2;
            end;
         2: begin
              ad_mode:= ((ad_mode + 1) mod 3);
            end;
         3: begin
              Write ('Enter offset to read from (decimal): ');
              Readln (line);
              addr:= StrToIntDef(line, 0);
              data:= 0;
              case ad_mode of
                ALTERA_MODE_BYTE:  begin
                                     data:= ALTERA_ReadByte(ALTERACard, ad_sp, addr);
                                   end;
                ALTERA_MODE_WORD:  begin
                                     data:= ALTERA_ReadWord(ALTERACard, ad_sp, addr);
                                   end;
                ALTERA_MODE_DWORD: begin
                                     data:= ALTERA_ReadDword(ALTERACard, ad_sp, addr);
                                   end
              end;
              Writeln (Format('Value read is: $%x',[data]));
            end;
         4: begin
              Write ('Enter offset to write to (decimal): ');
              Readln (line);
              addr:= StrToIntDef(line, 0);
              Write (Format('Enter data to write %s (hexadecimal): ',[ALTERA_GetActiveMode(ad_mode)]));
              Readln (line);
              if Copy (line, 1, 1) <> '$' then line:= '$' + line;
              try
                data:= StrToInt64(line);  // StrToInt64 considers string as hex and makes int from it
                case ad_mode of
                  ALTERA_MODE_BYTE:  begin
                                       ALTERA_WriteByte(ALTERACard, ad_sp, addr, Byte(data));
                                     end;
                  ALTERA_MODE_WORD:  begin
                                       ALTERA_WriteWord(ALTERACard, ad_sp, addr, Word(data));
                                     end;
                  ALTERA_MODE_DWORD: begin
                                       ALTERA_WriteDword(ALTERACard, ad_sp, addr, LongWord(data));
                                     end
                end;
              except
                on EConvertError do
                  Writeln ('Illegal input...');
              end;
            end;

    end (*case*)
  until cmd = 99;
end;

procedure ALTERA_EnableDisableInterrupts (ALTERACard: TkdCustomCard);
var
  cmd: Integer;
begin
  Writeln;
  WRITELN ('WARNING!!!');
  WRITELN ('==========');
  WRITELN ('YOUR HARDWARE HAS LEVEL-SENSITIVE INTERRUPT NATURE.');
  WRITELN ('YOU MUST MODIFY THE SOURCE CODE OF ALTERA_SpecifyInterruptHandling PROCEDURE');
  WRITELN ('IN Altera_Lib.pas UNIT TO ACKNOWLEDGE THE INTERRUPT BEFORE ENABLING INTERRUPTS.');
  WRITELN ('WITHOUT THIS MODIFICATION, YOUR PC WILL HANG UPON INTERRUPT!');

  repeat
    Writeln;
    Writeln ('Enable / Disable interrupts');
    Writeln ('---------------------------');
    Writeln (Format('1. %s Int',[ALTERA_GetIntPrompt(ALTERACard)]));
    Writeln ('99. Back to main menu');
    Writeln;
    Write ('Enter option: ');
    Readln(line);
    cmd:= StrToIntDef(line, 0);
    case cmd of
         1: begin
              if ALTERA_GetIntPrompt(ALTERACard) = 'Interrupt resource not found on board' then
              begin
                Writeln (ALTERA_GetIntPrompt(ALTERACard));
                cmd:= 99 //exit;
              end else
              if ALTERA_IntIsEnabled(ALTERACard) = True then
              begin
                Writeln ('Disabling interrupt Int');
                ALTERA_IntDisable(ALTERACard);
              end else
              begin
                Writeln ('Enabling interrupt Int');
                if not ALTERA_IntEnable (ALTERACard, ALTERAUserModeInterruptHandler.ALTERA_IntHandlerRoutine) then
                  Writeln ('Failed enabling interrupt Int')
              end;
            end;
    end
  until cmd = 99;
end;

function ALTERA_LocateAndOpenBoard(dwVendorId, dwDeviceId: LongWord): Boolean;
var
  cards, my_card: Longword;
begin
  Result:= False;
  my_card:= 0;
  cards:= ALTERA_CountCards (dwVendorId, dwDeviceId, ALTERACard.DevicePath+'/'+aux_device_file_name);
  if cards = 0 then
  begin
    Writeln (ALTERA_ErrorString);
    Exit;
  end else
  if cards = 1 then my_card:= 1 else
  begin
    Writeln (Format('Found %d matching ALTERA PCI cards',[cards]));
    Write (Format('Select card (1 - %d) ',[cards]));
    Readln (line);
    my_card:= StrToIntDef(line, 0);
    if (my_card < 1) or (my_card > cards) then
    begin
      Writeln ('Choice out of range');
      Exit;
    end
  end;

  if ALTERA_Open(ALTERACard, dwVendorId, dwDeviceId, my_card) then
  begin
    Writeln ('ALTERA PCI card found!');
    Result:= True
  end else Writeln (ALTERA_ErrorString);
end;

begin
  Writeln;
  Writeln ('ALTERA diagnostic utility.');
  Writeln ('Application accesses hardware using KylixDriver');
  ALTERAUserModeInterruptHandler:= TALTERAUserModeInterruptHandler.Create;
  ALTERADataModule:= TDataModule.Create(nil);
  try
    ALTERACard:= TkdCustomCard.Create(ALTERADataModule);
    try
      // If you are root, you can handle with loading the GDD, if need be.
      // Just correct these two lines then:
      // ALTERACard.DevicePath = '/dev';
      // ALTERACard.ScriptPath = '/home/eugene/kylixdriver/module/LINUX.2.4.20-8/';
      
      ALTERACard.ModuleLoaded:= True;
      if not ALTERACard.ModuleLoaded then
      begin
        Writeln ('The GDD is not loaded');
        Exit;
      end;

      if (ALTERA_DEFAULT_VENDOR_ID <> 0) then
        lobres:= ALTERA_LocateAndOpenBoard(ALTERA_DEFAULT_VENDOR_ID, ALTERA_DEFAULT_DEVICE_ID);
      repeat
        Writeln;
        Writeln('ALTERA main menu');
        Writeln('-----------------------------');
        Writeln('1. Scan PCI bus');
        Writeln('2. Locate/Choose ALTERA board');
        if (lobres) then
        begin
          Writeln('3. PCI configuration registers');
          Writeln('4. Access ALTERA memory and port ranges');
          Writeln('5. Enable / Disable interrupts');
        end;
        Writeln('99. Exit');
        Writeln;
        Write('Enter option: ');
        Readln(line);
        cmd:= StrToIntDef(line, 0);
        case cmd of
	     1: begin
	          PCI_Print_all_cards_info(ALTERACard.DevicePath+'/'+aux_device_file_name);
		end;
	     2: begin
	          { optionally unregister for new test }
	          if ALTERACard.CardRegistered then ALTERACard.UnregisterCard;
		  { optionally clear all resources to use the card object for new test }
		  ALTERACard.Clear;
		  lobres:= ALTERA_LocateAndOpenBoard(ALTERA_DEFAULT_VENDOR_ID, ALTERA_DEFAULT_DEVICE_ID);
		  if not lobres then
		    Writeln ('ALTERA card open failed!');
		end;
             3: begin
                  if lobres then
                    PCI_EditConfigReg(ALTERACard);
                end;
             4: begin
                  if lobres then
                    ALTERA_AccessRanges (ALTERACard);
                end;
             5: begin
                  if lobres then
                    ALTERA_EnableDisableInterrupts (ALTERACard);
                end;
	end
        until cmd = 99;

        ALTERA_Close(ALTERACard);  // may be unnecessary cos ALTERACard.Free unregisters car automatically

    finally
      ALTERACard.Free;
    end;
  finally
    ALTERADataModule.Free;
    ALTERAUserModeInterruptHandler.Free;
  end;
end.
