unit utils;

interface
uses
  kdclxcore;

  procedure PCI_Print_all_cards_info(AuxDevicePath: String);
  procedure PCI_EditConfigReg (ACard: TkdCustomCard);

implementation
uses
  Classes,
  SysUtils,
  kdframeworkassistant,
  kdpcihdr,
  kdpcitypes;

  procedure PrintCardInfo (FA: TkdFrameworkAssistant; PciConfig: pPciCfg);
  begin
    Writeln (Format('%s', [FA.PrintPciCardInfo(PciConfig)]));
    Write ('Press Enter to continue to next device');
    Readln;
  end;
  
  procedure PCI_Print_all_cards_info(AuxDevicePath: String);
  var
    i: Integer;
    FA: TkdFrameworkAssistant;
    PciCnt: Byte;
  begin
    FA:= TkdFrameworkAssistant.Create(AuxDevicePath);
    try
      with FA do
      begin
        PciCnt:= PciCount; { save it once, because FA.PciCount enumerates *all* devices }
        for i:= 1 to PciCnt do
          if PciConfig[i] <> nil then PrintCardInfo(FA, PciConfig[i]);
      end;
    finally
      FA.Free;
    end;
  end;

  procedure PCI_EditConfigReg (ACard: TkdCustomCard);
  var
    i, cmd, regcmd: Integer;
    line: String[255];
    ValueRead: String;
    APCINonBridge: TkdPCINonBridge;
  begin
    APCINonBridge:= nil;
    for i:= 0 to ACard.Count - 1 do
      if TComponent(ACard.CardItems[i]) is TkdPCINonBridge then
      begin
        APCINonBridge:= TkdPCINonBridge(ACard.CardItems[i]);
        Break;
      end;

    if not Assigned (APCINonBridge) then
    begin
      Writeln ('PCI configuration resource not found on board');
      Exit;
    end;
    
    repeat
      with APCINonBridge do
      begin
        Writeln;
        Writeln(Name,' access menu',' (Bus:',Bus,' Slot:',Slot,' Func:',Func,')');
        Writeln('---------------------------------------------------');
        Writeln('1.  Access CommandReg');
        Writeln('2.  Access StatusReg');
        Writeln('3.  Access RevisionId');
        Writeln('4.  Access ProgIf');
        Writeln('5.  Access SubClass');
        Writeln('6.  Access ClassCode');
        Writeln('7.  Access CacheLineSize');
        Writeln('8.  Access Latency');
        Writeln('9.  Access HeaderType');
        Writeln('10. Access BIST');
        Writeln('11. Access BaseAddr0');
        Writeln('12. Access BaseAddr1');
        Writeln('13. Access BaseAddr2');
        Writeln('14. Access BaseAddr3');
        Writeln('15. Access BaseAddr4');
        Writeln('16. Access BaseAddr5');
        Writeln('17. Access CardBusCIS');
        Writeln('18. Access ExpansionROM');
        Writeln('19. Access CapPtr');
        Writeln('20. Access InterruptLine');
        Writeln('21. Access InterruptPin');
        Writeln('22. Access MinGrant');
        Writeln('23. Access MaxLatency');
        Writeln('99. Exit');
        Write('Enter option: ');
        Readln(line);
        regcmd:= StrToIntDef(line,0);
        if (regcmd in [1..23]) then
        begin
          repeat
            Writeln;
            Writeln('1.  Read from the PCI configuration register');
            Writeln('2.  Write to the PCI configuration register');
            Writeln('99. Exit');
            Write('Enter option: ');
            Readln(line);
            cmd:= StrToIntDef(line,0);
            if cmd = 2 then
            begin
              Write ('Enter data to write (hexadecimal): ');
              Readln(line);
            end;
            { possible exceptions are handled by component itself }
            try
            try
              case regcmd of
                1:  begin
                      if cmd = 1 then ValueRead:= CommandReg else if cmd = 2 then CommandReg:= line
                    end;
                2:  begin
                      if cmd = 1 then ValueRead:= StatusReg else if cmd = 2 then StatusReg:= line
                    end;
                3:  begin
                      if cmd = 1 then ValueRead:= RevisionId else if cmd = 2 then RevisionId:= line
                    end;
                4:  begin
                      if cmd = 1 then ValueRead:= ProgIf else if cmd = 2 then ProgIf:= line
                    end;
                5:  begin
                      if cmd = 1 then ValueRead:= SubClass else if cmd = 2 then SubClass:= line
                    end;
                6:  begin
                      if cmd = 1 then ValueRead:= ClassCode else if cmd = 2 then ClassCode:= line
                    end;
                7:  begin
                      if cmd = 1 then ValueRead:= CacheLineSize else if cmd = 2 then CacheLineSize:= line
                    end;
                8:  begin
                      if cmd = 1 then ValueRead:= Latency else if cmd = 2 then Latency:= line
                    end;
                9:  begin
                      if cmd = 1 then ValueRead:= HeaderType else if cmd = 2 then HeaderType:= line
                    end;
                10: begin
                      if cmd = 1 then ValueRead:= BIST else if cmd = 2 then BIST:= line
                    end;
                11: begin
                      if cmd = 1 then ValueRead:= BaseAddr0 else if cmd = 2 then BaseAddr0:= line
                    end;
                12: begin
                      if cmd = 1 then ValueRead:= BaseAddr1 else if cmd = 2 then BaseAddr1:= line
                    end;
                13: begin
                      if cmd = 1 then ValueRead:= BaseAddr2 else if cmd = 2 then BaseAddr2:= line
                    end;
                14: begin
                      if cmd = 1 then ValueRead:= BaseAddr3 else if cmd = 2 then BaseAddr3:= line
                    end;
                15: begin
                      if cmd = 1 then ValueRead:= BaseAddr4 else if cmd = 2 then BaseAddr4:= line
                    end;
                16: begin
                      if cmd = 1 then ValueRead:= BaseAddr5 else if cmd = 2 then BaseAddr5:= line
                    end;
                17: begin
                      if cmd = 1 then ValueRead:= CardBusCIS else if cmd = 2 then CardBusCIS:= line
                    end;
                18: begin
                      if cmd = 1 then ValueRead:= ExpansionROM else if cmd = 2 then ExpansionROM:= line
                    end;
                19: begin
                      if cmd = 1 then ValueRead:= CapPtr else if cmd = 2 then CapPtr:= line
                    end;
                20: begin
                      if cmd = 1 then ValueRead:= InterruptLine else if cmd = 2 then InterruptLine:= line
                    end;
                21: begin
                      if cmd = 1 then ValueRead:= InterruptPin else if cmd = 2 then InterruptPin:= line
                    end;
                22: begin
                      if cmd = 1 then ValueRead:= MinGrant else if cmd = 2 then MinGrant:= line
                    end;
                23: begin
                      if cmd = 1 then ValueRead:= MaxLatency else if cmd = 2 then MaxLatency:= line
                    end;
              end;
              if (cmd in [1..2]) then
                if LastError = 0 then
                begin
                  if cmd = 1 then Writeln ('Hexadecimal value read is: ', ValueRead)
                    else Writeln('Data has been successfully written');
                end else Writeln ('Error accessing the PCI configuration register...');
            except
              on E: Exception do Writeln(E.Message);
            end
            except
              on E: Exception do Writeln(E.Message);
            end;
          until cmd = 99
        end
      end
    until regcmd = 99
  end;

end.

