unit Unit1;

interface

uses
  SysUtils, Types, Classes, Variants, QGraphics, QControls, QForms, QDialogs,
  kdregister, kdinterrupt, kdclxcore, kdranges, QStdCtrls;

type
  TForm1 = class(TForm)
    ShowTimeLabel: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Button1: TButton;
    ReadmeLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
uses
  DataModuleUnit;

{$R *.xfm}

type
  TCMOSTimeReadingThread = class (TThread)
  protected
    hours, minutes, seconds: String;
    FkdInterrupt: TkdInterrupt;
    FLabel: TLabel;
    procedure DisplayTime;
    procedure Execute; override;
  public
    constructor Create(AkdInterrupt: TkdInterrupt; ALabel: TLabel);
  end;

var
  CMOSTimeReadingThread: TCMOSTimeReadingThread;
  
procedure TForm1.FormCreate(Sender: TObject);
begin
  CustomCardDataModule.kdCustomCard1.ModuleLoaded:= True;
  if not CustomCardDataModule.kdCustomCard1.ModuleLoaded then
  begin
    ShowMessage ('The GDD is not loaded');
    Halt;
  end;

  CustomCardDataModule.kdCustomCard1.RegisterCard;
  if not CustomCardDataModule.kdCustomCard1.CardRegistered then
  begin
    ShowMessage ('Unable to register the card!');
    Halt;
  end;

  CMOSTimeReadingThread:= nil;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  CustomCardDataModule.kdinterrupt1.EnableInterrupt;
  if CMOSTimeReadingThread = nil then
    CMOSTimeReadingThread:= TCMOSTimeReadingThread.Create(CustomCardDataModule.kdInterrupt1, ShowTimeLabel);
  Button1.Caption:= 'Ready'
end;

constructor TCMOSTimeReadingThread.Create;
begin
  FkdInterrupt:= AkdInterrupt;
  FLabel:= ALabel;
  inherited Create(True);
  FreeOnTerminate:= False;
  Resume;
end;

procedure TCMOSTimeReadingThread.Execute;
begin
  repeat
    Synchronize(DisplayTime);
  until Terminated;
end;

procedure TCMOSTimeReadingThread.DisplayTime;
type
  pbyte = ^byte;
var
  bytepointer: pbyte;
begin
  if FkdInterrupt.SharedMemory > 0 then
  begin
    bytepointer:= pbyte(FkdInterrupt.SharedMemory);
    Inc(bytepointer, 2);  // points to hours
    hours:= IntToHex (bytepointer^, 2);  // getting hours from the page
    Dec(bytepointer);
    minutes:= IntToHex (bytepointer^, 2); // getting minutes
    Dec(bytepointer);
    seconds:= IntToHex (bytepointer^, 2);  // getting seconds  
    FLabel.Caption:= hours+':'+minutes+':'+seconds;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if CMOSTimeReadingThread <> nil then
  begin
    CMOSTimeReadingThread.Terminate;
    CMOSTimeReadingThread.WaitFor;
    FreeAndNil(CMOSTimeReadingThread);
    CustomCardDataModule.kdinterrupt1.DisableInterrupt;
  end;  
end;

end.
