/*
 * Main header file for KylixDriver's GDD module.
 *
 * This file may be distributed only as part of the 
 * application you are distributing, and only if it 
 * significantly contributes to the functionality of your 
 * application. (see kylixdriver.htm(l) for details).
 *
 * Web site: http://etsoftware.tripod.com/index.html
 * Email:    etsoft@tenzor.net
 *
 * (C) ETSoft Tenzor.Net 2002 - 2005
 */
 
/*
 * chardev.h -- definitions etc
 *
 */

#ifndef _CHARDEV_H_
#define _CHARDEV_H_

#include <stdarg.h>

/* there are two types of device files here */
#define   MAIN_DEVICE_NODE          0  /* responsible for functioning with device resources   */
#define   AUXILIARY_DEVICE_NODE     1  /* provides additional access to the hardware and other kernel job */

#define   TkdPortRange_RUID         1
#define   TkdMemoryRange_RUID       2
#define   TkdHWInterrupt_RUID       3
#define   TkdHWRangeRegister_RUID   4
#define   TkdDirectRPIOMRange_RUID  5
#define   TkdDirectRRAMRange_RUID   6
#define   TkdUSBDevice_RUID         7
#define   TkdPCINonBridge_RUID      8
#define   TkdDMABuffer_RUID         9

#define   max_resource_id           TkdDMABuffer_RUID

/* Use 'k' as magic number */
#define KYLIXDRIVER_IOC_MAGIC  'k'

#define KYLIXDRIVER_IOC_REGISTER_RESOURCE _IOWR (KYLIXDRIVER_IOC_MAGIC, 1, void*)
#define KYLIXDRIVER_IOC_UNREGISTER_RESOURCE _IOWR (KYLIXDRIVER_IOC_MAGIC, 2, int)
#define KYLIXDRIVER_IOC_HWRANGETRANSFER _IOWR (KYLIXDRIVER_IOC_MAGIC, 3, void*)
#define KYLIXDRIVER_IOC_READ_KDHWRANGEREGISTER _IOR (KYLIXDRIVER_IOC_MAGIC, 4, unsigned int*)
#define KYLIXDRIVER_IOC_WRITE_KDHWRANGEREGISTER _IOW (KYLIXDRIVER_IOC_MAGIC, 5, unsigned int*)

#define KYLIXDRIVER_IOC_GET_INTERRUPT_INFO _IOR(KYLIXDRIVER_IOC_MAGIC, 6, char*)
#define KYLIXDRIVER_IOC_ENABLE_INTERRUPT _IO(KYLIXDRIVER_IOC_MAGIC, 7)
#define KYLIXDRIVER_IOC_DISABLE_INTERRUPT _IO(KYLIXDRIVER_IOC_MAGIC, 8)

#define AUX_IOC_GET_LPT_BASE_BY_NUM _IOWR (KYLIXDRIVER_IOC_MAGIC, 9, void*)
#define AUX_IOC_GET_LPT_IRQ_BY_BASE _IOWR (KYLIXDRIVER_IOC_MAGIC, 10, void*)
#define AUX_IOC_GET_PCI_DEVS_INFO _IOWR (KYLIXDRIVER_IOC_MAGIC, 11, void*)
#define AUX_IOC_EXEC_NATIVE_KERNEL_ROUTINE _IOWR (KYLIXDRIVER_IOC_MAGIC, 16, void*)
#define AUX_IOC_CALL_USER_CODE_FROM_KERNEL _IOWR (KYLIXDRIVER_IOC_MAGIC, 17, void*)
#define AUX_IOC_SLEEP _IOWR (KYLIXDRIVER_IOC_MAGIC, 20, void*)

#define KYLIXDRIVER_IOC_GET_USB_INFO _IOR(KYLIXDRIVER_IOC_MAGIC, 12, char*)
#define KYLIXDRIVER_IOC_GET_USB_DEVICE_INFO _IOR(KYLIXDRIVER_IOC_MAGIC, 13, char*)

#define KYLIXDRIVER_IOC_READPCIREGISTER _IOWR(KYLIXDRIVER_IOC_MAGIC, 14, void*)
#define KYLIXDRIVER_IOC_WRITEPCIREGISTER _IOWR(KYLIXDRIVER_IOC_MAGIC, 15, void*)

#define KYLIXDRIVER_IOC_LOCKDMABUFFER _IOWR(KYLIXDRIVER_IOC_MAGIC, 18, void*)
#define KYLIXDRIVER_IOC_UNLOCKDMABUFFER _IOWR(KYLIXDRIVER_IOC_MAGIC, 19, void*)


struct semaphore;
struct usb_driver;
struct usb_device_id;
struct timeval;
struct __wait_queue_head;
struct __wait_queue;

typedef struct mutex_device {
   struct semaphore *mutex;
   void *device;
   struct semaphore *personal_interrupt_mutex;
} mutex_device;


enum hwinterrupt_interrupt_acknowledgement_actions {faREAD=0, faWRITE, faNONE};
enum hwinterrupt_interrupt_handler_types {htSlow=0, htFast};
enum hwinterrupt_interrupt_auto_request_modes {arCheckOnly=0, arInstall, arNone};
enum hwinterrupt_interrupt_types {itEdgeTriggered=0, itLevelSensitive};

typedef struct TkdInterruptStruct {
   unsigned int resource_unique_id;
   unsigned int resource_id;
   unsigned int AutoRequest;
   unsigned int InterruptNumber;
   unsigned int InterruptShared;
   unsigned int InterruptType;
   unsigned int HandlerType;
   unsigned int AcknowledgementData;
   unsigned int AcknowledgementItems;
   unsigned int UseInterruptAcknowledgement;
   unsigned int UseInterruptHandler;
   unsigned int HandlerStorage;
   unsigned int LinearSharedMemory; /* OUT */
} TkdInterruptStruct;

typedef struct TkdHWRangeStruct {
   unsigned int resource_unique_id;
   unsigned int resource_id;
   unsigned int RangeFrom;
   unsigned int RangeTo;
   unsigned int RangeShared;
   unsigned int MapType;
   unsigned int IORemappedAddr;     /* for TkdMemoryRange usage only ) */
} TkdHWRangeStruct;

typedef struct TkdDirectRangeStruct {
   unsigned int resource_unique_id;
   unsigned int resource_id;
   unsigned int RangeFrom;
   unsigned int RangeTo;
   unsigned int StartPageAddr;
   unsigned int MapLength;
   unsigned int MemProt;
   unsigned int MapType;
   unsigned int UserDirectAddr;
} TkdDirectRangeStruct;


enum TkdHWRangeRegister_use_modes {umRDONLY=0, umWRONLY, umRDWR};
enum TkdHWRangeRegister_sizes {rsBYTE=1, rsWORD, rsDWORD};
enum TkdHWRangeRegister_map_types {mtPORTMAPPED=0, mtMEMORYMAPPED};

typedef struct TkdHWRangeRegisterStruct {
   unsigned int resource_unique_id;
   unsigned int resource_id;
   unsigned int AutoRead;
   unsigned int RangeID;
   unsigned int Offset;
   unsigned int Size;
   unsigned int UseMode;
   unsigned int MapType;
} TkdHWRangeRegisterStruct;

typedef struct TkdUSBDeviceStruct {
   unsigned int resource_unique_id;
   unsigned int resource_id;
   unsigned int ProductId;
   unsigned int VendorId;
} TkdUSBDeviceStruct;


typedef struct TkdPCINonBridgeStruct {
   unsigned int resource_unique_id;
   unsigned int resource_id;
   unsigned int DeviceId;
   unsigned int VendorId;
   unsigned int Bus;
   unsigned int Slot;
   unsigned int Func;
   unsigned int RealBus;
   unsigned int RealSlot;
   unsigned int RealFunc;  
} TkdPCINonBridgeStruct;


typedef struct TkdHWRangeTransferStruct {
   unsigned int AutoInc;
   unsigned int TransferCount;
   unsigned int TransferDir;
   unsigned int TransferSize;
   unsigned int TransferOffset;
   unsigned int BufferAddr;
   unsigned int BufferSize;
   unsigned int ArrangedLike;
   unsigned int TransferWay;
} TkdHWRangeTransferStruct;

typedef struct TkdPCINonBridgeRegisterStruct {
   unsigned int RegOffset;
   unsigned int RegSize;
   unsigned int RegValue;
   unsigned int RegValue0;
} TkdPCINonBridgeRegisterStruct;

enum { KD_DMA_PAGES = 512 };

typedef struct
{
    unsigned long phys; // Physical address of page.
    unsigned long size; // Size of page.
} LINUX_dma_page;


enum TkdDMABuffer_options {boCONTIGUOUS = 0x1, // The system allocates a contiguous buffer.
        // user does not need to supply linear address.
                      boBELOW_16M = 0x2, // If boCONTIGUOUS is used,
        // this will make sure it is under 16M. Most recent kernels fail on it Why ?
		      reserved_option1 = 0x4, // do not touch
		      reserved_option2 = 0x8, // do not use
		      boKERNEL_ONLY_MAP = 0x10, // Only map to kernel, do not map to the user space.
        // relevant with boCONTIGUOUS flag only
		      boREAD_FROM_DEVICE = 0x20, // memory pages are locked to be written to (read from device)
		      boWRITE_TO_DEVICE = 0x40, // memory pages are locked to be read from (write to device)
};

typedef struct TkdDMABufferLockStruct {
   unsigned int UserAddr;
   unsigned int KernelAddr;
   unsigned int Size;
   unsigned int Options;
   unsigned int Pages;
   LINUX_dma_page PageArray [KD_DMA_PAGES];
} TkdDMABufferLockStruct;


typedef struct AUX_LPT_Num_Base_Struct {
   unsigned int Num;
   unsigned int Base;
} AUX_LPT_Num_Base_Struct;

typedef struct AUX_LPT_Base_Irq_Struct {
   unsigned int Base;
   unsigned int Irq;
} AUX_LPT_Base_Irq_Struct;

typedef struct AUX_SLEEP_Struct {
   unsigned long SleepMicrosecs;
   unsigned long SleepOptions;
} AUX_SLEEP_Struct;

enum Sleep_options {soNON_BUSY_SLEEP = 0x1
};

typedef struct usb_get_device_info_struct {
   unsigned int usb_dev_info_handle;
   unsigned int user_buffer;
} usb_get_device_info_struct;


typedef struct TkdHWRangeRegister_Dev {
    unsigned int resource_unique_id;
    unsigned int RangeID;
    unsigned int AddrToTransfer;
    unsigned int Size;
    unsigned int UseMode;
    unsigned int AutoRead;
    unsigned int MapType;
    struct mutex_device *pfile_context_mutex_device;
    unsigned int InAutoReadEntry;
} TkdHWRangeRegister_Dev;

typedef struct {
  struct TkdHWRangeRegister_Dev *dev;
  void *next;
} TkdHWRangeRegisterListNode;


typedef struct TkdRange_Dev {
    unsigned int resource_unique_id;
    unsigned int RangeFrom;
    unsigned int RangeTo;
    unsigned int RangeShared;
    unsigned long PhysRangeFrom;
    unsigned long RangeLength;
    unsigned int UserDirectAddr; /* for TkdDirectMemoryRange usage only */
    unsigned int MapLength;
    unsigned int MapType;
    TkdHWRangeRegisterListNode *TkdHWRangeRegistersListHead;
    struct semaphore* InAutoReadEntry_mutex;
} TkdRange_Dev;


/* internal instruction of the Virtual Interrupt Machine */
struct TkdInterrupt_Dev_insn {
    unsigned char opcode;
    unsigned char regnum;
    unsigned short arg16;
    unsigned int arg32; /* may be a char pointer */
};

enum opcodes {
    OPCODE_NONE = 0,
    OPCODE_READ,
    OPCODE_WRITE,
    OPCODE_MEMPAGE,
    OPCODE_MEMREAD8,
    OPCODE_MEMREAD16,
    OPCODE_MEMREAD32,
    OPCODE_MEMWRITE8,
    OPCODE_MEMWRITE16,
    OPCODE_MEMWRITE32,
    OPCODE_OR,
    OPCODE_AND,
    OPCODE_XOR,
    OPCODE_ADD,
    OPCODE_SUB,
    OPCODE_LOAD,
    OPCODE_STORE,
    OPCODE_PRINTK,
    OPCODE_JZ,
    OPCODE_JNZ,
    OPCODE_JMP,
    OPCODE_RET,
    OPCODE_SOI,
    OPCODE_MAX
};



/* structure used to pass interrupt handler data to kernel space */
struct TkdInterrupt_Dev_interrupt_handler_data {
    int insnlen, stringlen;
    struct TkdInterrupt_Dev_insn *insn;
    char *string;
};

/* state-machine for interrupt management */
struct TkdInterrupt_Dev_virtual_machine {
    unsigned long ip;
    unsigned int accumulator;
    unsigned int regs[15];
};

/* single action from the whole acknowledgement set of actions */
typedef struct TkdInterruptAcknowledgement {
    unsigned int AcknowledgementAction;
    unsigned int AcknowledgementRegister;
    unsigned int AcknowledgementValue;
} TkdInterruptAcknowledgement;

#define NR_TIMEVAL 2048 /* length of the array of time values */


typedef struct TkdInterrupt_Dev {
    unsigned int resource_unique_id;
    int IrqChannel;

    struct __wait_queue_head *_queue_;

    struct __wait_queue_head *disable_wait;

    struct __wait_queue *wait;
    unsigned long _buffer_;          /* not forget to init */
    unsigned long volatile _head_;
    volatile unsigned long _tail_;

    struct timeval *tv_data;
    volatile struct timeval *tv_head; /* not forget to init */
    volatile struct timeval *tv_tail; /* not forget to init */
    int count0;
    long enable_InterruptShared;
    long enable_InterruptType;
    long enable_HandlerType;
    struct TkdInterruptAcknowledgement *enable_AcknowledgementData;
    long enable_AcknowledgementItems;

    long enable_UseInterruptAcknowledgement;
    long enable_UseInterruptHandler;
    struct TkdInterrupt_Dev_interrupt_handler_data InterruptHandler;
    struct TkdInterrupt_Dev_virtual_machine VirtualMachine;
    unsigned long PhysicalSharedMemory;
    unsigned long LinearSharedMemory;

    unsigned short volatile last_wakeup;
    unsigned short volatile enabled;
    unsigned short AutoRequest;
    unsigned short volatile InSleepOn_queue_;
    short OurInterrupt;
    short TaskletIsScheduled;

} TkdInterrupt_Dev;


typedef struct {
  struct TkdInterrupt_Dev *dev;
  void *next;
} InterruptDevice;

typedef struct TkdPCINonBridge_Dev {
    unsigned int resource_unique_id;
    struct pci_dev *pcidev;

} TkdPCINonBridge_Dev;

typedef unsigned long DWORD, *PDWORD;

typedef struct kdUSB_Dev {
    unsigned int resource_unique_id;
    struct usb_driver *driver;

    struct __wait_queue_head *_queue_;
    struct __wait_queue_head *unvirtualize_wait;
    struct __wait_queue *wait;
    unsigned long _buffer_;
    unsigned long volatile _head_;
    volatile unsigned long _tail_;
    int count0;
    unsigned short volatile last_wakeup;
    unsigned short volatile enabled;
    unsigned short waitqueue_entry_initialized;
    unsigned short volatile InSleepOn_queue;
    DWORD active_config;
    DWORD active_ifnum;
    DWORD active_setting;
    void *device_info;
    DWORD device_info_size;
} kdUSB_Dev;

typedef struct TkdDMABuffer_Dev {
    unsigned int resource_unique_id;
    unsigned int PCIConfigID;              /* PCI device (related to SG buffer allocation only) */
    unsigned short unlock_needed;          /* set if DMA buffer is currently allocated*/
    unsigned long contiguous_mapped_addr;  /* user-mode linear address of the kernel-allocated buffer */
    unsigned long size;                    /* how many bytes with PAGE granularity are mapped for kernel-allocated buffer */
    void *contiguous_kvirt_addr;           /* kernel virtual address returned by kmalloc() of the kernel-allocated buffer */
    unsigned int options;
    void *dma_handle;                      /* related to user-allocated (SG) buffer */ 
    unsigned int initial_page_count;       /* related to SG buffer */
    unsigned int page_count;               /* related to SG buffer */
    unsigned int is_write_to_device;
} TkdDMABuffer_Dev;


typedef struct TkdDMABufferStruct {
    unsigned int resource_unique_id;
    unsigned int resource_id;
    unsigned int PCIConfigID;
} TkdDMABufferStruct;

#ifdef __KERNEL__


extern void TkdInterrupt_Dev_interpreter(struct TkdInterrupt_Dev *dev);
extern int read_io_register (struct TkdHWRangeRegister_Dev *kdHWRangeRegister, unsigned long arg, short is_user_buf, struct mutex_device *pmutex_device);
extern int write_io_register (struct TkdHWRangeRegister_Dev *kdHWRangeRegister, unsigned long arg, short is_user_buf, struct mutex_device *pmutex_device);

#endif /* __KERNEL__ */

#endif /* _CHARDEV_H_ */
