/* 
 * Source file for KylixDriver's GDD module.
 *
 * This file may be distributed only as part of the 
 * application you are distributing, and only if it 
 * significantly contributes to the functionality of your 
 * application. (see kylixdriver.htm(l) for details).
 *
 * Web site: http://etsoftware.tripod.com/index.html
 * Email:    etsoft@tenzor.net
 *
 * (C) ETSoft Tenzor.Net 2002 - 2005
 */
 
/*
 * interpreter.c -- the interpreter for interrupt handling scripts
 *
 */

#ifndef __KERNEL__
#  define __KERNEL__
#endif

#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/pci.h>
#include <asm/page.h>
#include <asm/io.h>

#include "chardev.h"
#include "sysdep.h"


#define staticfun static

static inline u32 getvalue(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    return insn->regnum
	? dev->VirtualMachine.regs[insn->regnum]
	: insn->arg32;
}

static inline void perform_mem_io(struct TkdInterrupt_Dev *dev, int regnum,
				  u32 addr, int dir, int size)
{
    u32 a;
    u32 mempage = dev->PhysicalSharedMemory;

    if (regnum) addr = dev->VirtualMachine.regs[regnum];

    addr &= ~PAGE_MASK; /* deal with overflows */
    if ( (addr + size) > PAGE_SIZE) return; /* error, keep it silent */

    if (dir) /* I */
	dev->VirtualMachine.accumulator = (size == 1)
	    ? *(u8 *)(mempage + addr)
	    : (size == 2)
	    ? *(u16 *)(mempage + addr)
	    : *(u32 *)(mempage + addr);
    else { /* O */
	a = dev->VirtualMachine.accumulator;
	if (size == 1) *(u8  *)(mempage + addr) = a & 0xff;
	if (size == 2) *(u16 *)(mempage + addr) = a & 0xffff;
	if (size == 4) *(u32 *)(mempage + addr) = a;
    }
}


static inline void perform_pci_io(struct TkdPCINonBridge_Dev *dev,
				  u32 addr, int dir, int size, struct TkdInterrupt_Dev *idev)
{
    u32 a; u16 b; u8 c;

    if (dir) {/* I */
	if (size == 1) {
	    LINUX_pci_read_config_byte(dev->pcidev, addr, &c);
	    idev->VirtualMachine.accumulator = c;
	}
	if (size == 2) {
	    LINUX_pci_read_config_word(dev->pcidev, addr, &b);
	    idev->VirtualMachine.accumulator = b;
	}
	if (size == 4) {
	    LINUX_pci_read_config_dword(dev->pcidev, addr, &a);
	    idev->VirtualMachine.accumulator = a;
	}
    } else { /* O */
	a = idev->VirtualMachine.accumulator;
	if (size == 1)
	    LINUX_pci_write_config_byte(dev->pcidev, addr, a & 0xff);
	if (size == 2)
	    LINUX_pci_write_config_word(dev->pcidev, addr, a & 0xffff);
	if (size == 4)
	    LINUX_pci_write_config_dword(dev->pcidev, addr, a);
    }
}


/* each opcode is implemented by a specific function */

staticfun int do_read(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    switch (*(u32*)insn->arg32) {
      case 4:  /* TkdHWRangeRegister object */

        dev->VirtualMachine.accumulator = 0;  // clear higher bits to grant actual value
        read_io_register ((struct TkdHWRangeRegister_Dev*)insn->arg32, (unsigned long)(&dev->VirtualMachine.accumulator), 1, NULL);
	break;

      case 8:  /* TkdPciNonBridge object */
        perform_pci_io((struct TkdPCINonBridge_Dev*)insn->arg32, insn->regnum, 1, insn->arg16, dev);
        break;

    }
    return 1;
}

staticfun int do_write(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    switch (*(u32*)insn->arg32) {
      case 4:   /* TkdHWRangeRegister object */

        write_io_register ((struct TkdHWRangeRegister_Dev*)insn->arg32, dev->VirtualMachine.accumulator, 0, NULL);
        break;

      case 8:   /* TkdPciNonBridge object */
        perform_pci_io((struct TkdPCINonBridge_Dev*)insn->arg32, insn->regnum /* offset*/, 0 /*in*/, insn->arg16 /*size*/, dev);
        break;

    }
    return 1;
}

staticfun int do_mempage(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.accumulator = dev->PhysicalSharedMemory;
    return 1;
}


staticfun int do_memread8(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    perform_mem_io(dev, insn->regnum, insn->arg32, 1 /* in */, 1);
    return 1;
}

staticfun int do_memread16(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    perform_mem_io(dev, insn->regnum, insn->arg32, 1 /* in */, 2);
    return 1;
}

staticfun int do_memread32(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    perform_mem_io(dev, insn->regnum, insn->arg32, 1 /* in */, 4);
    return 1;
}


staticfun int do_memwrite8(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    perform_mem_io(dev, insn->regnum, insn->arg32, 0 /* out */, 1);
    return 1;
}

staticfun int do_memwrite16(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    perform_mem_io(dev, insn->regnum, insn->arg32, 0 /* out */, 2);
    return 1;
}

staticfun int do_memwrite32(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    perform_mem_io(dev, insn->regnum, insn->arg32, 0 /* out */, 4);
    return 1;
}

staticfun int do_or(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.accumulator |= getvalue(dev, insn);
    return 1;
}

staticfun int do_and(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.accumulator &= getvalue(dev, insn);
    return 1;
}

staticfun int do_xor(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.accumulator ^= getvalue(dev, insn);
    return 1;
}

staticfun int do_add(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.accumulator += getvalue(dev, insn);
    return 1;
}

staticfun int do_sub(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.accumulator -= getvalue(dev, insn);
    return 1;
}

staticfun int do_load(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.accumulator = dev->VirtualMachine.regs[insn->regnum];
    return 1;
}

staticfun int do_store(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.regs[insn->regnum] = dev->VirtualMachine.accumulator;
    return 1;
}

extern const int printk_type;
extern struct {
    void (*printfun)(char* str);
} printk_fun_tab[];

staticfun int do_printk(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    char tmpbfr[100];
    if (strlen (dev->InterruptHandler.string + insn->arg32) < (100 - 10) )
      LINUX_sprintf (tmpbfr, dev->InterruptHandler.string + insn->arg32, dev->VirtualMachine.accumulator);
     else
      LINUX_sprintf (tmpbfr, "string too long at IP %li\n", dev->VirtualMachine.ip);

    printk_fun_tab[printk_type].printfun(tmpbfr);
    return 1;
}


staticfun int do_jmp(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->VirtualMachine.ip += insn->arg32;
    return 1;
}

staticfun int do_jz(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    if (!dev->VirtualMachine.accumulator)
	return do_jmp(dev, insn);
    return 1;
}

staticfun int do_jnz(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    if (dev->VirtualMachine.accumulator)
	return do_jmp(dev, insn);
    return 1;
}


staticfun int do_ret(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    return 0; /* return */
}

staticfun int do_soi(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn)
{
    dev->OurInterrupt = 1;
    return 0; /* return */
}



/* This is the dispatch table, it _must_ be in the same order as the opcodes */

struct {
    int (*fun)(struct TkdInterrupt_Dev *dev, struct TkdInterrupt_Dev_insn *insn);
} opcode_tab[] = {
    {NULL}, /* opcode 0 is not valid */
    {do_read},
    {do_write},
    {do_mempage},
    {do_memread8},
    {do_memread16},
    {do_memread32},
    {do_memwrite8},
    {do_memwrite16},
    {do_memwrite32},
    {do_or},
    {do_and},
    {do_xor},
    {do_add},
    {do_sub},
    {do_store},
    {do_load},
    {do_printk},
    {do_jz},
    {do_jnz},
    {do_jmp},
    {do_ret},
    {do_soi},
    {NULL}, /* I fear my errors */
};


void TkdInterrupt_Dev_interpreter(struct TkdInterrupt_Dev *dev)
{
    struct TkdInterrupt_Dev_insn *insn;
    char tmpbfr[100];
    dev->VirtualMachine.ip = 0;
    while (1) {
	insn = dev->InterruptHandler.insn + dev->VirtualMachine.ip;
	if (insn->opcode < 1 || insn->opcode >= OPCODE_MAX) {

	    LINUX_sprintf (tmpbfr, "invalid instruction at IP %li\n", dev->VirtualMachine.ip);
	    printk_fun_tab[printk_type].printfun(tmpbfr);
	    return;
	}
	if (! (opcode_tab[insn->opcode].fun(dev, insn)) )
	    break;

	dev->VirtualMachine.ip++;
	if (dev->VirtualMachine.ip >= dev->InterruptHandler.insnlen)
	    // end of instructions
	    break;
    }
}

