/* 
 * Source file for KylixDriver's GDD module.
 *
 * This file may be distributed only as part of the 
 * application you are distributing, and only if it 
 * significantly contributes to the functionality of your 
 * application. (see kylixdriver.htm(l) for details).
 *
 * Web site: http://etsoftware.tripod.com/index.html
 * Email:    etsoft@tenzor.net
 *
 * (C) ETSoft Tenzor.Net 2002 - 2005
 */

#ifndef __KERNEL__
#  define __KERNEL__
#endif

#include "kdcommon.h"
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/pci.h>
#include <asm/page.h>
#include <asm/io.h>

#include "sysdep.h"
#include "plugin1.h"

#  define staticfun  static



/* Consider using this file for adding your own kernel plugin functions with subsequent rebuilding the GDD */

u32 SoundOn(void)
{


__asm__ __volatile__ (  "in $0x61,%%ax\n"
			"\tor $0x03,%%ax\n"
			"\tout %%ax,$0x61\n"
			"\tmov $0xb6,%%ax\n"
			"\tout %%ax,$0x43\n"
			"\tmov $0xff,%%al\n"
			"\tout %%al,$0x42\n"
			"\tmov $0x02,%%al\n"
			"\tout %%al,$0x42\n"
			"\tnop" : : : "eax");
	return 0;

}


u32 SoundOff(void)
{
__asm__ __volatile__ ("in $0x61,%%ax\n"
                      "\tand $0xfc,%%ax\n"
		      "\tout %%ax,$0x61\n"
		      "\tmovl %%eax,%%eax" : : : "eax");
	return 0;
}


u32 Read_CMOS_result;
u8  secs, mins, hours;  /* I think these must be global, do not ask me why, I did not yet realized */
u32 Read_CMOS(void)
{

__asm__ __volatile__ (  "cli\n"
			"\tmov $0x70,%%dx\n"
			"\tmov $0x00,%%al\n"
			"\tout %%al,%%dx\n"
			"\tmov $0x71,%%dx\n"
			"\tin %%dx,%%al\n"
			#if defined(LINUX_24)
			"\tmov %%al," SYMBOL_NAME_STR(secs) "\n"
			#else
			"\tmov %%al,secs \n"
			#endif
			"\tmov $0x70,%%dx\n"
			"\tmov $0x02,%%al\n"
			"\tout %%al,%%dx\n"
			"\tmov $0x71,%%dx\n"
			"\tin %%dx,%%al\n"
			#if defined(LINUX_24)
			"\tmov %%al," SYMBOL_NAME_STR(mins) "\n"
			#else
			"\tmov %%al,mins\n"
			#endif
			"\tmov $0x70,%%dx\n"
			"\tmov $0x04,%%al\n"
			"\tout %%al,%%dx\n"
			"\tmov $0x71,%%dx\n"
			"\tin %%dx,%%al\n" 
			#if defined(LINUX_24)
			"\tmov %%al," SYMBOL_NAME_STR(hours) "\n"
			#else
			"\tmov %%al,hours\n"
			#endif
			"\tsti\n"
			"\tnop" : : : "eax");

        Read_CMOS_result = (hours<<16);
        Read_CMOS_result |= (mins<<8);
	Read_CMOS_result |= (secs);
	return Read_CMOS_result;

}



u32 defaultnkrresult = 0;
u32 defaultnkr(void)
{
	return defaultnkrresult;
}

extern unsigned int ExecAndReturn(unsigned int (*fn)(void) /*void * arg*/ /*,unsigned long flags*/);


typedef struct RoutineParams {
  u32 name; /* pointer char* to procedure name */
  u32 len;
} RoutineParams;

spinlock_t call_sl = SPIN_LOCK_UNLOCKED;

unsigned long ExecNativeKernelRoutine(unsigned long arg)
{
  #define IOC_BUFSIZE 128
  u32 result;
  static unsigned char localbuf[IOC_BUFSIZE];
  static struct RoutineParams routineparams;
  LINUX_spin_lock(&call_sl);
  LINUX_memset(localbuf, 0, IOC_BUFSIZE);
  LINUX_copy_from_user(&routineparams, (char*)arg, sizeof(RoutineParams));  /* let me be silent, check for errors yourself */
  LINUX_copy_from_user(localbuf, (char*)routineparams.name, routineparams.len);    /* let be in silent as well */


  if (strcmp(localbuf, "SoundOn") == 0) /* coincides with what user wants to execute */  result = ExecAndReturn(SoundOn);  else

  if (strcmp(localbuf, "SoundOff") == 0) /* coincides with what user wants to execute */  result = ExecAndReturn(SoundOff);  else

  if (strcmp(localbuf, "Read_CMOS") == 0) /* coincides with what user wants to execute */  result = ExecAndReturn(Read_CMOS);  else

  /* and so on, so on ... */

  result = ExecAndReturn(defaultnkr);
  LINUX_spin_unlock(&call_sl);
  return result;
}

