/*
 * Source file for KylixDriver's GDD module.
 *
 * This file may be distributed only as part of the
 * application you are distributing, and only if it
 * significantly contributes to the functionality of your
 * application. (see kylixdriver.htm(l) for details).
 *
 * Web site: http://etsoftware.tripod.com/index.html
 * Email:    etsoft@tenzor.net
 *
 * (C) ETSoft Tenzor.Net 2002 - 2005
 */

#include "kdcommon.h"

#include <linux/pci.h>
#include <linux/usb.h>
#if defined(LINUX_24_26)
    #include <linux/interrupt.h>
#endif


#include <linux/tty.h>
#include <linux/tty_driver.h>

#include <linux/string.h>
#include <linux/types.h>
#include <linux/signal.h>
#include <linux/errno.h>

#if defined(LINUX_24)
#include <linux/tqueue.h>
#include <assert.h>
#endif

#include <asm/system.h>
#include <asm/semaphore.h>
#include <asm/uaccess.h>

#include <linux/delay.h>
#if defined(LINUX_26)
    #include <linux/pagemap.h>
    #include <asm/cacheflush.h>
    #if defined(LINUX_2610)
      #include <asm/pgtable.h>
    #endif  
#endif
#if defined(LINUX_24)
    #include <linux/iobuf.h>
    #include <linux/vmalloc.h>
#endif

#include <asm/mman.h>
#include "sysdep.h"
#if defined(KYLIXDRIVER_KERNEL) && defined(LINUX_24)
    #define EXPORT_SYMTAB
#endif
#include <linux/module.h> // must come after #define EXPORT_SYMTAB

#if defined(MODVERSIONS)
    #include <linux/modversions.h>
#endif

#if defined(MODULE_LICENSE)
    MODULE_LICENSE("Proprietary");
#endif

MODULE_AUTHOR (DRIVER_AUTHOR);
MODULE_DESCRIPTION (DRIVER_DESC);

#if defined(LINUX_24) || defined(LINUX_26)

#include <linux/slab.h>


#define vm_offset vm_pgoff

spinlock_t mmap_spin_lock, call_user_code_from_kernel_sl, sleep_sl = SPIN_LOCK_UNLOCKED;
rwlock_t access_user_processes_list_rwl = RW_LOCK_UNLOCKED;

unsigned int addresses[] = {
    PCI_BASE_ADDRESS_0,
    PCI_BASE_ADDRESS_1,
    PCI_BASE_ADDRESS_2,
    PCI_BASE_ADDRESS_3,
    PCI_BASE_ADDRESS_4,
    PCI_BASE_ADDRESS_5,
    0
};

void *LINUX_kmalloc(unsigned int size, int flag) /*was int dma instead of flag*/
{
    if (size > 128*1024)
        return NULL;
    flag |= flag & ATOMIC ? GFP_ATOMIC : GFP_KERNEL;
    flag |= flag & DMA ? GFP_DMA : 0;
    return kmalloc((size_t) size, flag);
}

void *LINUX_kmalloc_GFP_KERNEL(unsigned int size)
{
    return LINUX_kmalloc (size, GFP_KERNEL);
}

void *LINUX_kmalloc_GFP_USER(unsigned int size)
{
    return LINUX_kmalloc (size, GFP_USER);
}

void *LINUX_kmalloc_GFP_DMA(unsigned int size)
{
    return LINUX_kmalloc (size, GFP_KERNEL|GFP_DMA);
}

void *LINUX_kmalloc_GFP_ATOMIC(unsigned int size)
{
    return LINUX_kmalloc (size, GFP_ATOMIC);
}

void LINUX_kfree(void *addr)
{
    kfree((const void *)addr);
}

void *LINUX_vmalloc(unsigned long size)
{
    return vmalloc (size);
}

void LINUX_vfree(void * addr)
{
    vfree(addr);
}

unsigned long LINUX_copy_from_user(void * to, const void * from, unsigned long n)
{
    return copy_from_user (to, from, n);
}

unsigned long LINUX_copy_to_user(void * to, const void * from, unsigned long n)
{
   return copy_to_user (to, from, n);
}

void *LINUX_memcpy(void *to, const void *from, unsigned long n)
{
   return memcpy(to, from, n);
}

void *LINUX_memset(void *addr, int s, unsigned long count)
{
    return memset(addr, s, count);
}

#if defined(KYLIXDRIVER_KERNEL)

    struct file_operations kylixdriver_fops = {
        ioctl:  device_ioctl,
        open: device_open,
        release: device_release,
	mmap: device_mmap,
    };

    int LINUX_register_chrdev(unsigned int major, const char * name)
    {
        return register_chrdev(major, name, &kylixdriver_fops);
    }

    int LINUX_unregister_chrdev(unsigned int major, const char * name)
    {
        return unregister_chrdev(major, name);
    }
#endif

extern void do_tasklet_interrupt (int, void*, struct pt_regs*);
#if defined(LINUX_26)
irqreturn_t
#else
void
#endif
LINUX_tasklet_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
    do_tasklet_interrupt (irq, dev_id, regs);
    #if defined(LINUX_26)
    return 0;
    #endif
}

int LINUX_request_irq(unsigned int irq,
    unsigned long flags, const char *device, void *dev_id)
{
    return request_irq(irq, LINUX_tasklet_interrupt, flags, device, dev_id);
}

void LINUX_free_irq(unsigned int irq, void *dev_id)
{
    free_irq(irq, dev_id);
}


void * LINUX_ioremap (unsigned long phys_addr, unsigned long size)
{

    return (void *) ioremap(phys_addr, size);
}

void LINUX_iounmap(void *addr)
{
    iounmap(addr);
}

unsigned long LINUX_do_mmap_pgoff(struct file * file, unsigned long addr, unsigned long len,
    unsigned long prot, unsigned long flags, unsigned long off)
{
    int res;
#if defined (LINUX_26)
    down_write(&current->mm->mmap_sem);
#endif

    res = do_mmap_pgoff (file, addr, len, prot, flags, off);

#if defined (LINUX_26)
    up_write(&current->mm->mmap_sem);
#endif
    return res;
}

int LINUX_do_munmap(unsigned long addr, unsigned int len)
{
    return do_munmap(current->mm, addr, (size_t) len
#if defined(DO_MUNMAP_API_CHANGE)
    , 0
#endif
    );
}

unsigned long LINUX_do_mmap_shared_page(struct file * file, unsigned long ouraddr)
{
    return LINUX_do_mmap_pgoff(file, 0, PAGE_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, (__pa(ouraddr)) >> PAGE_SHIFT);
}

unsigned long LINUX_do_mmap_kernelbuffer(struct file * file, unsigned long addr, unsigned long size)
{
    return LINUX_do_mmap_pgoff(file, 0, size, PROT_READ | PROT_WRITE, MAP_SHARED, addr);
}

struct page *LINUX_virt_to_page (void * kaddr)
{
    return virt_to_page(kaddr);
}

unsigned long LINUX_jiffies()
{
    return jiffies;
}

unsigned long LINUX_HZ() 
{
    return HZ;
}

void *LINUX_high_memory()
{
    return (void*)high_memory;
}

int LINUX_remap_page_range(struct vm_area_struct *vma, unsigned long virt_add, unsigned long phys_add, unsigned long size)
{
#if defined (LINUX_2610)
    return io_remap_page_range(vma, virt_add, phys_add, size, vma->vm_page_prot);
#else    
    return remap_page_range(
#if defined(REMAP_API_CHANGE) || defined(LINUX_26)
        vma,
#endif
virt_add, phys_add, size, vma->vm_page_prot);
#endif
}


void LINUX_printk(char *str)
{
     struct tty_struct *my_tty;
     my_tty = current
     #if defined(LINUX_266)
     -> signal
     #endif
     -> tty;

     if (my_tty !=NULL) {
       (*(my_tty->driver)
           #if defined(LINUX_24)
           .
	   #else
	   ->
	   #endif
           write) (
           my_tty,
	   #if defined(LINUX_2610)
	   #else
           0,
	   #endif
           str,
           strlen(str)
       );
       
 
     (*(my_tty-> driver)
          #if defined(LINUX_24)
          .
	  #else
	  ->
	  #endif
          write) (
          my_tty,
	  #if defined(LINUX_2610)
	  #else
          0,
	  #endif
          "\015\012",
          2);
     }

}

int LINUX_native_printk(const char *fmt, ...)
{
    int p[12];
    int res;
    int i;
    va_list args;
    va_start(args, fmt);
    for (i=0; i<sizeof(p)/sizeof(*p); i++)
        p[i] = va_arg(args, int);
    res = printk(fmt, p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7], p[8], p[9], p[10], p[11]);
    va_end(args);
    return res;
}

int LINUX_sprintf(char *buf, const char *fmt, ...)
{
    int res;
    va_list args;
    va_start(args, fmt);
    res = vsprintf(buf, fmt, args);
    va_end(args);
    return res;
}


void LINUX_wake_up_interruptible (void *queue)
{
    wake_up_interruptible((wait_queue_head_t*)queue);
}

void LINUX_wake_up (void *q)
{
    wake_up((wait_queue_head_t*)q);
}


void LINUX_interruptible_sleep_on_timeout (void *queue, long timeout)
{
    interruptible_sleep_on_timeout((wait_queue_head_t*)queue, timeout * HZ);
}


void LINUX_do_gettimeofday (struct timeval *tv)
{
    do_gettimeofday (tv);
}

void LINUX_free_pages (unsigned long addr, unsigned long order)
{
    free_pages(addr, order);
}

void LINUX_free_page (unsigned long addr)
{
    free_page(addr);
}

unsigned long LINUX___get_free_pages (int flags, unsigned long order)
{
    return __get_free_pages (flags, order);
}

unsigned long LINUX___get_free_pages_GFP_KERNEL (unsigned long order)
{
    return __get_free_pages (GFP_KERNEL, order);
}


unsigned long LINUX_get_free_page (int flags)
{
    #if defined(LINUX_24)
    return get_free_page(flags);
    #else
    return __get_free_pages(flags,0);
    #endif
}

unsigned long LINUX_get_free_page_GFP_USER (void)
{
    return LINUX_get_free_page (GFP_USER);
}

void LINUX_release_region (unsigned long from, unsigned long extent)
{
    release_region (from, extent);
}

void LINUX_release_mem_region (unsigned long from, unsigned long extent)
{
    release_mem_region (from, extent);
}

struct resource *LINUX_request_region (unsigned long from, unsigned long extent, const char *name)
{
    return request_region (from, extent, name);
}

struct resource *LINUX_request_mem_region (unsigned long start, unsigned long extent, const char *name)
{
    return request_mem_region (start, extent, name);
}

int LINUX_check_mem_region (unsigned long start, unsigned long extent)
{
    return check_mem_region (start, extent);
}

void LINUX_tasklet_schedule (struct tasklet_struct *tasklet)
{
    tasklet_schedule (tasklet);
}

void LINUX_add_wait_queue (void *q, void *wait)
{
    add_wait_queue ((wait_queue_head_t*)q, (wait_queue_t*)wait);
}

void LINUX_remove_wait_queue (void *q, void *wait)
{
    remove_wait_queue ((wait_queue_head_t*)q, (wait_queue_t*)wait);
}

void LINUX_schedule(void)
{
    schedule();
}

unsigned long LINUX_probe_irq_on (void)
{
    return probe_irq_on();
}

int LINUX_probe_irq_off (unsigned long mask)
{
    return probe_irq_off (mask);
}

void LINUX_udelay(unsigned long usecs)
{
    udelay(usecs);
}

int LINUX_pcibios_present(void)
{
#if defined(LINUX24) 
    return pcibios_present();
#else
    return 1;   
#endif
}

void LINUX_pci_enable_device(struct pci_dev *dev)
{
    pci_enable_device(dev);
}

struct pci_dev *LINUX_pci_find_slot (unsigned int bus, unsigned int devfn)
{
    struct pci_dev *dev;

    dev = pci_find_slot (bus, devfn);

    return dev;
}

struct pci_dev *LINUX_pci_find_device (unsigned int vendor, unsigned int device, const struct pci_dev *from)
{
    return pci_find_device (vendor, device, from);
}

int LINUX_pci_read_config_byte (struct pci_dev *dev, int where, u8 *val)
{
    return pci_read_config_byte (dev, where, val);
}

int LINUX_pci_read_config_word (struct pci_dev *dev, int where, u16 *val)
{
    return pci_read_config_word (dev, where, val);
}

int LINUX_pci_read_config_dword (struct pci_dev *dev, int where, u32 *val)
{
    return pci_read_config_dword (dev, where, val);
}

int LINUX_pci_write_config_byte (struct pci_dev *dev, int where, u8 val)
{
    return pci_write_config_byte (dev, where, val);
}

int LINUX_pci_write_config_word (struct pci_dev *dev, int where, u16 val)
{
    return pci_write_config_word (dev, where, val);
}

int LINUX_pci_write_config_dword (struct pci_dev *dev, int where, u32 val)
{
    return pci_write_config_dword (dev, where, val);
}


void LINUX_mdelay (unsigned long msecs)
{
    mdelay(msecs);
}

void LINUX_spin_lock_init(void *lock)
{
    spin_lock_init((spinlock_t*)lock);
}

void LINUX_spin_lock(void *lock)
{
    spin_lock((spinlock_t*)lock);
}

void LINUX_spin_unlock(void *lock)
{
    spin_unlock((spinlock_t*)lock);
}

void LINUX_read_lock (void *lock)
{
    read_lock ((rwlock_t*)lock);
}

void LINUX_read_unlock (void *lock)
{
    read_unlock ((rwlock_t*)lock);
}

void LINUX_write_lock_bh (void *lock)
{
    write_lock_bh ((rwlock_t*)lock);
}

void LINUX_write_unlock_bh (void *lock)
{
    write_unlock_bh ((rwlock_t*)lock);
}

void LINUX_cli(void)
{
    cli();
}

void LINUX_sti(void)
{
    sti();
}


int LINUX_down_interruptible(struct semaphore *sem)
{
    return down_interruptible(sem);
}

void LINUX_up(struct semaphore *sem)
{
    up(sem);
}

int LINUX_MOD_INC_USE_COUNT(void)
{
#if defined(LINUX_24)
    MOD_INC_USE_COUNT;
    return 1;
#else
    struct module *thismodule = THIS_MODULE;
    return try_module_get (thismodule);
#endif
}

void LINUX_MOD_DEC_USE_COUNT(void)
{
#if defined(LINUX_24)
    MOD_DEC_USE_COUNT;
#else
    struct module *thismodule = THIS_MODULE;
    module_put (thismodule);        
#endif
}

extern struct page *do_kylixdriver_vma_nopage(struct vm_area_struct*, unsigned long);
#if defined(LINUX_26)
struct page *kylixdriver_vma_nopage(struct vm_area_struct *vma,
    unsigned long address, int *write) 
#else
struct page *kylixdriver_vma_nopage(struct vm_area_struct *vma,
    unsigned long address, int write) 
#endif
{
    return do_kylixdriver_vma_nopage(vma, address);
}


struct vm_operations_struct simple_nopage_vm_ops = {
    nopage:  kylixdriver_vma_nopage,
};

extern int mmap_call_result;

int device_mmap(struct file *filp, struct vm_area_struct *vma)
{
    unsigned long offset = vma->vm_pgoff << PAGE_SHIFT;

    if (offset >= __pa(LINUX_high_memory) || (filp->f_flags & O_SYNC))
        vma->vm_flags |= VM_IO;
    vma->vm_flags |= VM_RESERVED;

    vma->vm_ops = &simple_nopage_vm_ops;

    if (LINUX_remap_page_range(vma, vma->vm_start, offset, vma->vm_end-vma->vm_start))
        return -EAGAIN;

    mmap_call_result = 0;  // everything as to mapping to user space happened Ok

    return 0;
}

int FillPCIDeviceName (void* buf, struct pci_dev *pcidev)
{
    return LINUX_sprintf (buf, "%s", pcidev->
    #if defined(LINUX_24)
    name
    #else
    	#if defined(CONFIG_PCI_NAMES)
	    pretty_name
	#else	
	    dev.bus_id
	#endif
    #endif
    );
}

void LINUX_init_waitqueue_head(void *q)
{
    init_waitqueue_head((wait_queue_head_t*)q);
}

void LINUX_init_waitqueue_entry (void *q, struct task_struct *cur)
{
    init_waitqueue_entry((wait_queue_t*)q, cur);
}

void LINUX_init_waitqueue_entry_current (void *q)
{
    init_waitqueue_entry((wait_queue_t*)q, current);
}

u32 LINUX__put_user(u32 datum, u32 *ptr)
{
    return __put_user(datum, ptr);
}

void LINUX__get_user(u32 *datum, u32 *ptr)
{
    __get_user(*datum, ptr);
}

int LINUX_read_access_ok (void * address, int size)
{
    return access_ok (VERIFY_READ, address, size);
}

unsigned char LINUX_inb(unsigned short port)
{
    return inb(port);
}

unsigned short LINUX_inw(unsigned short port)
{
    return inw(port);
}

unsigned int LINUX_inl(unsigned short port)
{
    return inl(port);
}

unsigned char LINUX_inb_p(unsigned short port)
{
    return inb_p(port);
}

unsigned short LINUX_inw_p(unsigned short port)
{
    return inw_p(port);
}

unsigned int LINUX_inl_p(unsigned short port)
{
    return inl_p(port);
}


void LINUX_outb(unsigned char value, unsigned short port)
{
    outb(value, port);
}

void LINUX_outw(unsigned short value, unsigned short port)
{
    outw(value, port);
}

void LINUX_outl(unsigned int value, unsigned short port)
{
    outl(value, port);
}

void LINUX_outb_p(unsigned char value, unsigned short port)
{
    outb_p(value, port);
}

void LINUX_outw_p(unsigned short value, unsigned short port)
{
    outw_p(value, port);
}

void LINUX_outl_p(unsigned int value, unsigned short port)
{
    outl_p(value, port);
}


void LINUX_insb(unsigned short port, void *addr, unsigned long count)
{
    insb(port, addr, count);
}
void LINUX_insw(unsigned short port, void *addr, unsigned long count)
{
    insw(port, addr, count);
}

void LINUX_insl(unsigned short port, void *addr, unsigned long count)
{
    insl(port, addr, count);
}

void LINUX_outsb(unsigned short port, const void *addr, unsigned long count)
{
    outsb(port, addr, count);
}

void LINUX_outsw(unsigned short port, const void *addr, unsigned long count)
{
    outsw(port, addr, count);
}

void LINUX_outsl(unsigned short port, const void *addr, unsigned long count)
{
    outsl(port, addr, count);
}

unsigned char LINUX_readb (unsigned int addr)
{   
    #if defined(LINUX_269)
    return readb((const volatile void __iomem*) addr);
    #else
    return readb(addr);
    #endif
    
 /* Not allowed: directives may not be used inside a macro argument
    return readb(
    #if defined(LINUX_269)
    (const volatile void __iomem*)
    #endif
    addr); */
}

unsigned short LINUX_readw (unsigned int addr)
{
    #if defined(LINUX_269)
    return readw((const volatile void __iomem*) addr);
    #else
    return readw(addr);
    #endif
}

unsigned int LINUX_readl (unsigned int addr)
{
    #if defined(LINUX_269)
    return readl((const volatile void __iomem*) addr);
    #else
    return readl(addr);
    #endif
}

void LINUX_writeb (unsigned char data, unsigned int addr)
{
    #if defined(LINUX_269)
    writeb (data,(volatile void __iomem*) addr);
    #else
    writeb (data, addr);
    #endif
}

void LINUX_writew (unsigned short data, unsigned int addr)
{
    #if defined(LINUX_269)
    writew (data,(volatile void __iomem*) addr);
    #else
    writew (data, addr);
    #endif
}

void LINUX_writel (unsigned int data, unsigned int addr)
{
    #if defined(LINUX_269)
    writel (data,(volatile void __iomem*) addr);
    #else
    writel (data, addr);
    #endif
}

void LINUX_rmb (void)
{
    rmb();
}

void LINUX_wmb (void)
{
    wmb();
}

void LINUX_mb (void)
{
    mb();
}

#if defined(LINUX_USB_SUPPORT)

int LINUX_usb_register (struct usb_driver *driver)
{
   return usb_register (driver);
}

void LINUX_usb_deregister (struct usb_driver *driver)
{
   usb_deregister (driver);
}

void test1(void)
{
   printk(KERN_EMERG "usb: raced timeout\n ");
}

struct usb_interface *LINUX_usb_ifnum_to_if(struct usb_device *dev, unsigned ifnum)
{
   return (struct usb_interface *)usb_ifnum_to_if(dev, ifnum);
}

int LINUX_usb_set_interface(struct usb_device *dev, int interface, int alternate)
{
   return usb_set_interface(dev, interface, alternate);
}

#endif

void* KD_get_usb_id_table (kdUSB_Dev *kdusb_dev)
{
    return (void*)kdusb_dev->driver->id_table;
}

int KD_get_usb_device_id_size (void)
{
    return sizeof(struct usb_device_id);
}

int KD_get_usb_driver_size(void)
{
    return sizeof(struct usb_driver);
}

struct usb_driver *KD_get_usb_driver(kdUSB_Dev *kdusb_dev)
{
    return kdusb_dev->driver;
}

void KD_assign_usb_driver_name (kdUSB_Dev *kdusb_dev, const char* driver_name)
{
    kdusb_dev->driver->name = driver_name;
}

void KD_assign_usb_driver_id_table (kdUSB_Dev *kdusb_dev, struct usb_device_id *id)
{
    kdusb_dev->driver->id_table = id;
}

void KD_assign_matching_params (struct usb_device_id *id, unsigned int VendorId, unsigned int ProductId, kdUSB_Dev *kdusb_dev)
{
    id[0].match_flags = USB_DEVICE_ID_MATCH_VENDOR | USB_DEVICE_ID_MATCH_PRODUCT;
    id[0].idVendor = VendorId;
    id[0].idProduct = ProductId;
    id[0].driver_info = (unsigned long) kdusb_dev;
}


void KD_do_pci_magic (struct pci_dev *pcidev)
{
    unsigned short pcicmnd;
    LINUX_pci_read_config_word(pcidev, PCI_COMMAND, &pcicmnd);
    pcicmnd |= PCI_COMMAND_IO | PCI_COMMAND_MEMORY;
    LINUX_pci_write_config_word(pcidev, PCI_COMMAND, pcicmnd);
}

unsigned int KD_get_pci_region_size(unsigned int mask)
{
    unsigned int size;
    if (mask & PCI_BASE_ADDRESS_SPACE_IO) {
      mask &= PCI_BASE_ADDRESS_IO_MASK;
      size = (~mask + 1) & 0xffff; /* Bleah */
    } else {
        mask &= PCI_BASE_ADDRESS_MEM_MASK;
        size = ~mask + 1;
    }
    return size;
}

unsigned short KD_get_pci_vendor(struct pci_dev *pcidev)
{
    return pcidev->vendor;
}

unsigned short KD_get_pci_device(struct pci_dev *pcidev)
{
    return pcidev->device;
}

unsigned short KD_get_pci_bus(struct pci_dev *pcidev)
{
    return pcidev->bus->number;
}

unsigned short KD_get_pci_slot(struct pci_dev *pcidev)
{
    return ((pcidev->devfn)>>3);
}

unsigned short KD_get_pci_func(struct pci_dev *pcidev)
{
    return ((pcidev->devfn)&7);
}




unsigned char KD_get_pci_header_type(struct pci_dev *pcidev)
{
    unsigned char headertype;
    LINUX_pci_read_config_byte(pcidev, PCI_HEADER_TYPE, &headertype);
    return headertype;
}

struct mutex_device* KD_get_mutex_device_from_file(struct file *filp)
{ 
    return filp->private_data;
}

void KD_set_mutex_device_to_file (struct file *filp, mutex_device *pmutex_device)
{
    filp->private_data = pmutex_device;
}

int KD_get_minor_from_node (struct inode *inode)
{
    return MINOR(inode->i_rdev) ;
}    
 
unsigned long KD_get_physaddr (struct vm_area_struct *vma, unsigned long address)
{ 
    return address - vma->vm_start + (vma->vm_pgoff << PAGE_SHIFT);  
}   

void LINUX_get_page(struct page *pageptr)
{
    get_page(pageptr);
}

int LINUX_signal_pending (struct task_struct* ts)
{
    return signal_pending (ts);
}

int LINUX_signal_pending_current (void)
{
    return signal_pending (current);
}

char* LINUX_strcat (char* str1, char* str2)
{
    return strcat (str1, str2);
}

void LINUX_set_current_state_TASK_INTERRUPTIBLE (void)
{
    set_current_state (TASK_INTERRUPTIBLE);
}

void LINUX_set_current_state_TASK_RUNNING (void)
{
    set_current_state (TASK_RUNNING);
}

unsigned long KD_enable_interrupt_get_flags(struct TkdInterrupt_Dev *kdInterrupt)
{
    return (kdInterrupt->enable_InterruptShared ? SA_SHIRQ : 0) | ((kdInterrupt->enable_HandlerType == htFast)?SA_INTERRUPT : 0 );
}


extern void TkdInterrupt_Dev_do_tasklet (unsigned long);


#define taskletdecl(x,y) \
  DECLARE_TASKLET ((x), TkdInterrupt_Dev_do_tasklet, (y));

taskletdecl(tasklet_int0, 0)
taskletdecl(tasklet_int1, 1)
taskletdecl(tasklet_int2, 2)
taskletdecl(tasklet_int3, 3)
taskletdecl(tasklet_int4, 4)
taskletdecl(tasklet_int5, 5)
taskletdecl(tasklet_int6, 6)
taskletdecl(tasklet_int7, 7)
taskletdecl(tasklet_int8, 8)
taskletdecl(tasklet_int9, 9)
taskletdecl(tasklet_int10, 10)
taskletdecl(tasklet_int11, 11)
taskletdecl(tasklet_int12, 12)
taskletdecl(tasklet_int13, 13)
taskletdecl(tasklet_int14, 14)
taskletdecl(tasklet_int15, 15)

void KD_all_tasklets_schedule (struct TkdInterrupt_Dev* kdInterrupt)
{
    switch (kdInterrupt->IrqChannel) {

        case 0:
            LINUX_tasklet_schedule(&tasklet_int0);
            break;
        case 1:
            LINUX_tasklet_schedule(&tasklet_int1);
            break;
        case 2:
            LINUX_tasklet_schedule(&tasklet_int2);
            break;
        case 3:
            LINUX_tasklet_schedule(&tasklet_int3);
            break;
        case 4:
            LINUX_tasklet_schedule(&tasklet_int4);
            break;
        case 5:
            LINUX_tasklet_schedule(&tasklet_int5);
            break;
        case 6:
            LINUX_tasklet_schedule(&tasklet_int6);
            break;
        case 7:
            LINUX_tasklet_schedule(&tasklet_int7);
            break;
        case 8:
            LINUX_tasklet_schedule(&tasklet_int8);
            break;
        case 9:
            LINUX_tasklet_schedule(&tasklet_int9);
            break;
        case 10:
            LINUX_tasklet_schedule(&tasklet_int10);
            break;
        case 11:
            LINUX_tasklet_schedule(&tasklet_int11);
            break;
        case 12:
            LINUX_tasklet_schedule(&tasklet_int12);
            break;
        case 13:
            LINUX_tasklet_schedule(&tasklet_int13);
            break;
        case 14:
            LINUX_tasklet_schedule(&tasklet_int14);
            break;
        case 15:
            LINUX_tasklet_schedule(&tasklet_int15);
            break;
    }
}

#define circularbuffer \
    if (*tvp == (tvd + NR_TIMEVAL - 1)) \
        *tvp = tvd; else (*tvp)++;


void KD_short_incr_tv(volatile struct timeval **tvp, struct timeval * tvd)
{
    circularbuffer
}

int KD_get_addapted_volatile_timeval_tv_sec (volatile struct timeval *tv)
{
    return (int)((tv)->tv_sec % 100000000);
}



int KD_get_volatile_timeval_tv_usec (volatile struct timeval *tv)
{
    return (int)((tv)->tv_usec);
}

struct timeval * KD_kmalloc_tv_data (void)
{
    return LINUX_kmalloc_GFP_KERNEL(sizeof(struct timeval) * NR_TIMEVAL);
}

struct timeval * KD_kmalloc_timeval (void)
{
    return LINUX_kmalloc_GFP_KERNEL(sizeof(struct timeval));
}


struct semaphore * KD_kmalloc_semaphore (void)
{
    return LINUX_kmalloc_GFP_KERNEL(sizeof(struct semaphore));
}

void LINUX_init_MUTEX (struct semaphore *mutex)
{
    init_MUTEX(mutex);
}

void *KD_get_access_user_processes_list_rwl(void)
{
    return &access_user_processes_list_rwl;
}

void *KD_get_call_user_code_from_kernel_sl(void)
{
    return &call_user_code_from_kernel_sl;
}

void *KD_get_sleep_sl(void)
{
    return &sleep_sl;
}

void *KD_get_mmap_spin_lock(void)
{
    return &mmap_spin_lock;
}

struct __wait_queue_head * KD_kmalloc__wait_queue_head (void)
{
    return LINUX_kmalloc_GFP_KERNEL(sizeof(struct __wait_queue_head));
}

struct __wait_queue * KD_kmalloc__wait_queue (void)
{
    return LINUX_kmalloc_GFP_KERNEL(sizeof(struct __wait_queue));
}

void *KD_get_c_NULL(void)
{
    return NULL;
}

unsigned int KD_get_PAGE_SIZE(void)
{
    return PAGE_SIZE;
}

unsigned int KD_get_PAGE_MASK(void)
{
    return PAGE_MASK;
}

unsigned long LINUX_PAGE_ALIGN(unsigned long addr)
{
    return PAGE_ALIGN (addr);
}

unsigned char KD_get_PAGE_SHIFT(void)
{
    return PAGE_SHIFT;
}

void * LINUX__va(unsigned long addr)
{
    return __va(addr);
}

unsigned long LINUX_kvirt_to_phys(unsigned long adr)
{
    unsigned long ul;
    ul = virt_to_phys((void *)adr);
    return ul;
}


#if defined(LINUX_26)
static int build_dma_list_26(LINUX_dma_page *page_list, void *buf,
    unsigned int page_count, int is_write_to_device, void *dev_handle, void **dma_handle)
{
    int rc, res = 0, i, j;
    struct page **pages = NULL;
    struct scatterlist *sgl = NULL;

    sgl = kmalloc(sizeof(struct scatterlist) * page_count, GFP_KERNEL);

    if (!sgl)
    {
        rc = -ENOMEM;
        goto Error;
    }
    pages = kmalloc(page_count * sizeof(*pages), GFP_ATOMIC);

    if (!pages)
    {
        rc= -ENOMEM;
        goto Error;
    }
    /* Try to fault in all of the necessary pages */
    down_read(&current->mm->mmap_sem);

    res = get_user_pages(current, current->mm, (unsigned long)buf,
        page_count, is_write_to_device , 0, /* don't force */ pages, NULL);

    up_read(&current->mm->mmap_sem);

    /* no pages was mapped */
    if (res < page_count)
    {
        rc = -1;
        goto Error;
    }

    memset (sgl, 0, sizeof(struct scatterlist) * page_count);
    for (i=0; i < page_count ; i++)
    {
        sgl[i].page = pages[i];
        sgl[i].length = PAGE_SIZE;
    }

    pci_map_sg((struct pci_dev *)dev_handle, sgl, page_count,
        is_write_to_device ? PCI_DMA_TODEVICE : PCI_DMA_FROMDEVICE);
    for (i=0; i<page_count; i++)
    {
        page_list[i].phys = sg_dma_address(&sgl[i]);
        page_list[i].size = sg_dma_len(&sgl[i]);
    }
    kfree(pages);
    *dma_handle = sgl;
    return 0;

 Error:
    if (res > 0)
    {
        for (j=0; j < res; j++)
            page_cache_release(pages[j]);
    }
    if (pages)
        kfree(pages);
    if (sgl)
        kfree(sgl);

    return rc;
}
#endif

#if defined(LINUX_24)
static int build_dma_list_24(LINUX_dma_page *page_list, void *buf,   unsigned long size, int is_write_to_device, void *dev_handle, void **dma_handle)
{
    int rc, i;
    struct kiobuf *iobuf;
    char buftemp [100];
#if defined(KIOBUF_WITH_SIZE)
    int nbhs = KIO_MAX_SECTORS;
    rc = alloc_kiovec_sz(1, &iobuf ,&nbhs);
#else
    rc = alloc_kiovec(1, &iobuf);
#endif

    if (rc)
    {
        LINUX_printk("failed allocate iobuf");
        goto Error;
    }

    rc = map_user_kiobuf(READ, iobuf, (unsigned long) buf, size);
    if (rc)
    {
        LINUX_sprintf(buftemp, "failed lock user buffer %p, size %ld, is_write_to_device %s\n",
            buf, size, is_write_to_device ? "true" : "false");
        LINUX_printk (buftemp);
        goto Error;
    }

    for (i=0; i<iobuf->nr_pages; i++)
    {
	page_list[i].phys = (iobuf->maplist[i] - mem_map) * PAGE_SIZE;
        page_list[i].size = PAGE_SIZE;
    }
    *dma_handle= iobuf;
    return 0;

Error:
#if defined(KIOBUF_WITH_SIZE)
    free_kiovec_sz(1, &iobuf, &nbhs);
#else
    free_kiovec(1, &iobuf);
#endif
    return rc;

}
#endif

int KD_get_initial_page_count (unsigned long size, void *buf)
{
    return (((unsigned long)buf & ~PAGE_MASK) + size - 1 + ~PAGE_MASK)
        >> PAGE_SHIFT;

}

int LINUX_build_sg_dma(LINUX_dma_page *page_list, unsigned long *page_count,
    unsigned long size, int is_write_to_device, void *buf, void *dev_handle,
    void **dma_handle)
{
    unsigned int nr_pages;
    int rc = 0;

    nr_pages = (((unsigned long)buf & ~PAGE_MASK) + size - 1 + ~PAGE_MASK)
        >> PAGE_SHIFT;

#if defined(LINUX_26)
    rc =  build_dma_list_26(page_list, buf, nr_pages, is_write_to_device, dev_handle,
        dma_handle);
#elif defined(LINUX_24)
    rc =  build_dma_list_24(page_list, buf, size, is_write_to_device, dev_handle,
        dma_handle);
#else
    // Not suppored by older kernels
    rc = 0x2000000aL;
#endif
    if (!rc)
        *page_count = nr_pages;
    return rc;
}

int LINUX_free_sg_dma(void *dma_handle, unsigned long page_count, int
is_write_to_device,
    void *dev_handle)
{
    int rc = 0;
#if defined(LINUX_26)
    int i;
    struct scatterlist *sgl = (struct scatterlist *)dma_handle;

    pci_unmap_sg((struct pci_dev *)dev_handle, sgl, page_count,
        is_write_to_device ? PCI_DMA_TODEVICE : PCI_DMA_FROMDEVICE);

    for (i=0; i < page_count; i++)
    {
        if (!PageReserved(sgl[i].page))
            SetPageDirty(sgl[i].page);
        page_cache_release(sgl[i].page);
    }
#elif defined(LINUX_24)
#if defined(KIOBUF_WITH_SIZE)
    int nbhs = KIO_MAX_SECTORS;
#endif
    struct kiobuf *iobuf = (struct kiobuf *)dma_handle;
    if (iobuf)
    {
        unmap_kiobuf(iobuf);
#if defined(KIOBUF_WITH_SIZE)
        free_kiovec_sz(1, &iobuf, &nbhs);
#else
        free_kiovec(1, &iobuf);
#endif
    }
#else
    // not supported by older kernels
    rc = 0x2000000aL;
#endif
    return rc;
}


/* Initialize the module - Register the character device */
static int kdmodule_init(void)
{
    return c_kdmodule_init();
}


/* Cleanup - unregister the appropriate file from /proc */
static void kdmodule_exit(void)
{
    c_kdmodule_exit();
}



/*#ifdef CONFIG_DEBUG_IOVIRT
void * __io_virt_deb ug(unsigned long x, const char *file, int line)
{
        char tmpbfr[100];

	if (x < PAGE_OFFSET) {
		LINUX_sprintf (tmpbfr, "io mapaddr 0x%05lx not valid at %s:%d!\n", x, file, line);
		return __va(x);
	}
	return (void *)x;
}
#endif*/

module_init (kdmodule_init);
module_exit (kdmodule_exit);


#endif


