/*
 * Header file for KylixDriver's GDD module.
 *
 * This file may be distributed only as part of the 
 * application you are distributing, and only if it 
 * significantly contributes to the functionality of your 
 * application. (see kylixdriver.htm(l) for details).
 *
 * Web site: http://etsoftware.tripod.com/index.html
 * Email:    etsoft@tenzor.net
 *
 * (C) ETSoft Tenzor.Net 2002 - 2005
 */

#ifndef _SYSDEP_H_
#define _SYSDEP_H_

#include <stdarg.h>
#include "chardev.h"

#define ATOMIC 0x1
#define DMA    0x2
struct semaphore;
struct pt_regs;
struct inode;
struct file;
struct vm_area_struct;
struct timeval;
struct tasklet_struct;
struct task_struct;

#if defined(LINUX_24)

#endif

#if defined(__cplusplus)
    extern "C" {
#endif

#ifndef BZERO
    #define BZERO(buf) LINUX_memset(&(buf), 0, sizeof(buf))
#endif

#define DRIVER_VERSION "Version 3.0"
#define DRIVER_AUTHOR "Copyright (c) 2002 - 2005 ETSoft Tenzor.Net"
#define DRIVER_DESC "KylixDriver - Sophisticated Kylix ToolKit for Hardware Access, GDD module."

//-----------------------------------------------------------------------------------------------------
int device_open(struct inode *inode, struct file *filp);
int device_release(struct inode *inode, struct file *filp);
int device_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);
int device_mmap(struct file *filp, struct vm_area_struct *vma);

//-----------------------------------------------------------------------------------------------------
int c_kdmodule_init(void);
void c_kdmodule_exit(void);

//-----------------------------------------------------------------------------------------------------
void *LINUX_kmalloc(unsigned int size, int dma);
void *LINUX_kmalloc_GFP_KERNEL(unsigned int size);
void *LINUX_kmalloc_GFP_USER(unsigned int size);
void *LINUX_kmalloc_GFP_DMA(unsigned int size);
void *LINUX_kmalloc_GFP_ATOMIC(unsigned int size);
void LINUX_kfree(void *addr);
void *LINUX_vmalloc(unsigned long size);
void LINUX_vfree(void *addr);
unsigned long LINUX_copy_from_user(void * to, const void * from, unsigned long n);
unsigned long LINUX_copy_to_user(void * to, const void * from, unsigned long n);
void *LINUX_memcpy(void *to, const void *from, unsigned long n);
void *LINUX_memset(void *addr ,int s, unsigned long count);
int LINUX_register_chrdev(unsigned int major, const char * name);
int LINUX_unregister_chrdev(unsigned int major, const char * name);
int LINUX_request_irq(unsigned int irq,
               unsigned long flags,
               const char *device,
               void *dev_id);
void LINUX_free_irq(unsigned int irq, void *dev_id);
void *LINUX_ioremap (unsigned long phys_addr, unsigned long size);
void LINUX_iounmap(void *addr);
unsigned long LINUX_do_mmap_pgoff(struct file * file, unsigned long addr, unsigned long len,
    unsigned long prot, unsigned long flags, unsigned long off);
unsigned long LINUX_do_mmap_shared_page(struct file * file, unsigned long ouraddr);
unsigned long LINUX_do_mmap_kernelbuffer(struct file * file, unsigned long addr, unsigned long size);
int LINUX_do_munmap(unsigned long addr, unsigned int len);
struct page *LINUX_virt_to_page (void * kaddr);
unsigned long LINUX_jiffies(void);
unsigned long LINUX_HZ(void);
void *LINUX_high_memory(void);
int LINUX_remap_page_range(struct vm_area_struct *vma, unsigned long virt_add,unsigned long phys_add, unsigned long size);
void LINUX_printk(char *str);
int LINUX_native_printk(const char *fmt, ...);
int LINUX_sprintf(char *buf, const char *fmt, ...);
void LINUX_wake_up_interruptible (void *queue);
void LINUX_wake_up (void *q);
void LINUX_interruptible_sleep_on_timeout (void *queue, long timeout);
void LINUX_do_gettimeofday (struct timeval *tv);
void LINUX_free_pages (unsigned long addr, unsigned long order);
void LINUX_free_page (unsigned long addr);
unsigned long LINUX___get_free_pages (int flags, unsigned long order);
unsigned long LINUX___get_free_pages_GFP_KERNEL (unsigned long order);
unsigned long LINUX_get_free_page (int flags);
unsigned long LINUX_get_free_page_GFP_USER (void);
void LINUX_release_region (unsigned long from, unsigned long extent);
void LINUX_release_mem_region (unsigned long from, unsigned long extent);
struct resource *LINUX_request_region (unsigned long from, unsigned long extent, const char *name);
struct resource *LINUX_request_mem_region (unsigned long start, unsigned long extent, const char *name);
int LINUX_check_mem_region (unsigned long start, unsigned long extent);
void LINUX_tasklet_schedule (struct tasklet_struct *tasklet);
void LINUX_add_wait_queue (void *q, void *wait);
void LINUX_remove_wait_queue (void *q, void *wait);
void LINUX_schedule(void);
unsigned long LINUX_probe_irq_on (void);
int LINUX_probe_irq_off (unsigned long mask);
void LINUX_udelay(unsigned long usecs);
int LINUX_pcibios_present(void);
struct pci_dev *LINUX_pci_find_slot (unsigned int bus, unsigned int devfn);
struct pci_dev *LINUX_pci_find_device (unsigned int vendor, unsigned int device, const struct pci_dev *from);
void LINUX_pci_enable_device(struct pci_dev *dev);
int LINUX_pci_read_config_byte (struct pci_dev *dev, int where, unsigned char *val);
int LINUX_pci_read_config_word (struct pci_dev *dev, int where, unsigned short *val);
int LINUX_pci_read_config_dword (struct pci_dev *dev, int where, unsigned int *val);
int LINUX_pci_write_config_byte (struct pci_dev *dev, int where, unsigned char val);
int LINUX_pci_write_config_word (struct pci_dev *dev, int where, unsigned short val);
int LINUX_pci_write_config_dword (struct pci_dev *dev, int where, unsigned int val);
void LINUX_mdelay (unsigned long msecs);
void LINUX_spin_lock_init(void *lock);
void LINUX_spin_lock (void *lock);
void LINUX_spin_unlock (void *lock);
void LINUX_read_lock (void *lock);
void LINUX_read_unlock (void *lock);
void LINUX_write_lock_bh (void *lock);
void LINUX_write_unlock_bh (void *lock);
void LINUX_cli (void);
void LINUX_sti (void);
int LINUX_down_interruptible(struct semaphore * sem);
void LINUX_up(struct semaphore * sem);
int LINUX_MOD_INC_USE_COUNT(void);
void LINUX_MOD_DEC_USE_COUNT(void);
void LINUX_init_waitqueue_head(void *q);
void LINUX_init_waitqueue_entry (void *q, struct task_struct *cur);
void LINUX_init_waitqueue_entry_current (void *q);
unsigned int LINUX__put_user (unsigned int datum, unsigned int *ptr);
void LINUX__get_user (unsigned int *datum, unsigned int *ptr);
int LINUX_read_access_ok (void * address, int size);
unsigned char LINUX_inb(unsigned short port);
unsigned short LINUX_inw(unsigned short port);
unsigned int LINUX_inl(unsigned short port);
unsigned char LINUX_inb_p(unsigned short port);
unsigned short LINUX_inw_p(unsigned short port);
unsigned int LINUX_inl_p(unsigned short port);
void LINUX_outb(unsigned char value, unsigned short port);
void LINUX_outw(unsigned short value, unsigned short port);
void LINUX_outl(unsigned int value, unsigned short port);
void LINUX_outb_p(unsigned char value, unsigned short port);
void LINUX_outw_p(unsigned short value, unsigned short port);
void LINUX_outl_p(unsigned int value, unsigned short port);
void LINUX_insb(unsigned short port, void *addr, unsigned long count);
void LINUX_insw(unsigned short port, void *addr, unsigned long count);
void LINUX_insl(unsigned short port, void *addr, unsigned long count);
void LINUX_insw(unsigned short port, void *addr, unsigned long count);
void LINUX_outsb(unsigned short port, const void *addr, unsigned long count);
void LINUX_outsw(unsigned short port, const void *addr, unsigned long count);
void LINUX_outsl(unsigned short port, const void *addr, unsigned long count);
unsigned char LINUX_readb (unsigned int addr);
unsigned short LINUX_readw (unsigned int addr);
unsigned int LINUX_readl (unsigned int addr);
void LINUX_writeb (unsigned char data, unsigned int addr);
void LINUX_writew (unsigned short data, unsigned int addr);
void LINUX_writel (unsigned int data, unsigned int addr);
void LINUX_rmb (void);
void LINUX_wmb (void);
void LINUX_mb (void);
void LINUX_get_page(struct page *pageptr);
int LINUX_signal_pending (struct task_struct* ts);
int LINUX_signal_pending_current (void);
char * LINUX_strcat (char* str1, char* str2);
void LINUX_set_current_state_TASK_INTERRUPTIBLE (void);
void LINUX_set_current_state_TASK_RUNNING (void);
void LINUX_init_MUTEX (struct semaphore *mutex);
void * LINUX__va(unsigned long addr);
unsigned long LINUX_PAGE_ALIGN(unsigned long addr);
unsigned long LINUX_kvirt_to_phys(unsigned long adr);
int LINUX_build_sg_dma(LINUX_dma_page *page_list, unsigned long *page_count,
    unsigned long size, int is_write_to_device, void *buf, void *dev_handle,
    void **dma_handle);
int LINUX_free_sg_dma(void *handle, unsigned long pages, int is_write_to_device,
    void *dev_handle);


//-----------------------------------------------------------------------------------------------------
void* KD_get_usb_id_table (kdUSB_Dev *kdusb_dev);
int KD_get_usb_device_id_size(void);
int KD_get_usb_driver_size(void);
struct usb_driver *KD_get_usb_driver(kdUSB_Dev *kdusb_dev);
void KD_assign_usb_driver_name (kdUSB_Dev *kdusb_dev, const char* driver_name);
void KD_assign_usb_driver_id_table (kdUSB_Dev *kdusb_dev, struct usb_device_id *id);
void KD_assign_matching_params (struct usb_device_id *id, unsigned int VendorId, unsigned int ProductId, kdUSB_Dev *kdusb_dev);
void KD_do_pci_magic (struct pci_dev *pcidev);
unsigned int KD_get_pci_region_size(unsigned int mask);
unsigned short KD_get_pci_vendor(struct pci_dev *pcidev);
unsigned short KD_get_pci_device(struct pci_dev *pcidev);
unsigned short KD_get_pci_bus(struct pci_dev *pcidev);
unsigned short KD_get_pci_slot(struct pci_dev *pcidev);
unsigned short KD_get_pci_func(struct pci_dev *pcidev);
unsigned char KD_get_pci_header_type(struct pci_dev *pcidev);
struct mutex_device* KD_get_mutex_device_from_file(struct file *filp);
void KD_set_mutex_device_to_file (struct file *filp, mutex_device *pmutex_device);
int KD_get_minor_from_node (struct inode *inode);
unsigned long KD_get_physaddr (struct vm_area_struct *vma, unsigned long address);
unsigned long KD_enable_interrupt_get_flags(struct TkdInterrupt_Dev *kdInterrupt);
void KD_all_tasklets_schedule (struct TkdInterrupt_Dev* kdInterrupt);
void KD_short_incr_tv(volatile struct timeval **tvp, struct timeval * tvd);
int KD_get_addapted_volatile_timeval_tv_sec (volatile struct timeval *tv);
int KD_get_volatile_timeval_tv_usec (volatile struct timeval *tv);
struct timeval * KD_kmalloc_tv_data (void);
struct timeval * KD_kmalloc_timeval (void);
struct semaphore * KD_kmalloc_semaphore (void);
void *KD_get_access_user_processes_list_rwl(void);
void *KD_get_call_user_code_from_kernel_sl(void);
void *KD_get_mmap_spin_lock(void);
void *KD_get_sleep_sl(void);
struct __wait_queue_head * KD_kmalloc__wait_queue_head (void);
struct __wait_queue * KD_kmalloc__wait_queue (void);
void *KD_get_c_NULL(void);
unsigned int KD_get_PAGE_SIZE(void);
unsigned int KD_get_PAGE_MASK(void);
unsigned char KD_get_PAGE_SHIFT(void);
int KD_get_initial_page_count (unsigned long size, void *buf);



#if defined(__cplusplus)
    }
#endif

#endif //_SYSDEP_H_


